#!/usr/bin/python
# -*- coding: utf-8 -*-
s = """
abaco
abbaglio
abbinato
abete
abisso
abolire
abrasivo
abrogato
accadere
accenno
accusato
acetone
achille
acido
acqua
acre
acrilico
acrobata
acuto
adagio
addebito
addome
adeguato
aderire
adipe
adottare
adulare
affabile
affetto
affisso
affranto
aforisma
afoso
africano
agave
agente
agevole
aggancio
agire
agitare
agonismo
agricolo
agrumeto
aguzzo
alabarda
alato
albatro
alberato
albo
albume
alce
alcolico
alettone
alfa
algebra
aliante
alibi
alimento
allagato
allegro
allievo
allodola
allusivo
almeno
alogeno
alpaca
alpestre
altalena
alterno
alticcio
altrove
alunno
alveolo
alzare
amalgama
amanita
amarena
ambito
ambrato
ameba
america
ametista
amico
ammasso
ammenda
ammirare
ammonito
amore
ampio
ampliare
amuleto
anacardo
anagrafe
analista
anarchia
anatra
anca
ancella
ancora
andare
andrea
anello
angelo
angolare
angusto
anima
annegare
annidato
anno
annuncio
anonimo
anticipo
anzi
apatico
apertura
apode
apparire
appetito
appoggio
approdo
appunto
aprile
arabica
arachide
aragosta
araldica
arancio
aratura
arazzo
arbitro
archivio
ardito
arenile
argento
argine
arguto
aria
armonia
arnese
arredato
arringa
arrosto
arsenico
arso
artefice
arzillo
asciutto
ascolto
asepsi
asettico
asfalto
asino
asola
aspirato
aspro
assaggio
asse
assoluto
assurdo
asta
astenuto
astice
astratto
atavico
ateismo
atomico
atono
attesa
attivare
attorno
attrito
attuale
ausilio
austria
autista
autonomo
autunno
avanzato
avere
avvenire
avviso
avvolgere
azione
azoto
azzimo
azzurro
babele
baccano
bacino
baco
badessa
badilata
bagnato
baita
balcone
baldo
balena
ballata
balzano
bambino
bandire
baraonda
barbaro
barca
baritono
barlume
barocco
basilico
basso
batosta
battuto
baule
bava
bavosa
becco
beffa
belgio
belva
benda
benevole
benigno
benzina
bere
berlina
beta
bibita
bici
bidone
bifido
biga
bilancia
bimbo
binocolo
biologo
bipede
bipolare
birbante
birra
biscotto
bisesto
bisnonno
bisonte
bisturi
bizzarro
blando
blatta
bollito
bonifico
bordo
bosco
botanico
bottino
bozzolo
braccio
bradipo
brama
branca
bravura
bretella
brevetto
brezza
briglia
brillante
brindare
broccolo
brodo
bronzina
brullo
bruno
bubbone
buca
budino
buffone
buio
bulbo
buono
burlone
burrasca
bussola
busta
cadetto
caduco
calamaro
calcolo
calesse
calibro
calmo
caloria
cambusa
camerata
camicia
cammino
camola
campale
canapa
candela
cane
canino
canotto
cantina
capace
capello
capitolo
capogiro
cappero
capra
capsula
carapace
carcassa
cardo
carisma
carovana
carretto
cartolina
casaccio
cascata
caserma
caso
cassone
castello
casuale
catasta
catena
catrame
cauto
cavillo
cedibile
cedrata
cefalo
celebre
cellulare
cena
cenone
centesimo
ceramica
cercare
certo
cerume
cervello
cesoia
cespo
ceto
chela
chiaro
chicca
chiedere
chimera
china
chirurgo
chitarra
ciao
ciclismo
cifrare
cigno
cilindro
ciottolo
circa
cirrosi
citrico
cittadino
ciuffo
civetta
civile
classico
clinica
cloro
cocco
codardo
codice
coerente
cognome
collare
colmato
colore
colposo
coltivato
colza
coma
cometa
commando
comodo
computer
comune
conciso
condurre
conferma
congelare
coniuge
connesso
conoscere
consumo
continuo
convegno
coperto
copione
coppia
copricapo
corazza
cordata
coricato
cornice
corolla
corpo
corredo
corsia
cortese
cosmico
costante
cottura
covato
cratere
cravatta
creato
credere
cremoso
crescita
creta
criceto
crinale
crisi
critico
croce
cronaca
crostata
cruciale
crusca
cucire
cuculo
cugino
cullato
cupola
curatore
cursore
curvo
cuscino
custode
dado
daino
dalmata
damerino
daniela
dannoso
danzare
datato
davanti
davvero
debutto
decennio
deciso
declino
decollo
decreto
dedicato
definito
deforme
degno
delegare
delfino
delirio
delta
demenza
denotato
dentro
deposito
derapata
derivare
deroga
descritto
deserto
desiderio
desumere
detersivo
devoto
diametro
dicembre
diedro
difeso
diffuso
digerire
digitale
diluvio
dinamico
dinnanzi
dipinto
diploma
dipolo
diradare
dire
dirotto
dirupo
disagio
discreto
disfare
disgelo
disposto
distanza
disumano
dito
divano
divelto
dividere
divorato
doblone
docente
doganale
dogma
dolce
domato
domenica
dominare
dondolo
dono
dormire
dote
dottore
dovuto
dozzina
drago
druido
dubbio
dubitare
ducale
duna
duomo
duplice
duraturo
ebano
eccesso
ecco
eclissi
economia
edera
edicola
edile
editoria
educare
egemonia
egli
egoismo
egregio
elaborato
elargire
elegante
elencato
eletto
elevare
elfico
elica
elmo
elsa
eluso
emanato
emblema
emesso
emiro
emotivo
emozione
empirico
emulo
endemico
enduro
energia
enfasi
enoteca
entrare
enzima
epatite
epilogo
episodio
epocale
eppure
equatore
erario
erba
erboso
erede
eremita
erigere
ermetico
eroe
erosivo
errante
esagono
esame
esanime
esaudire
esca
esempio
esercito
esibito
esigente
esistere
esito
esofago
esortato
esoso
espanso
espresso
essenza
esso
esteso
estimare
estonia
estroso
esultare
etilico
etnico
etrusco
etto
euclideo
europa
evaso
evidenza
evitato
evoluto
evviva
fabbrica
faccenda
fachiro
falco
famiglia
fanale
fanfara
fango
fantasma
fare
farfalla
farinoso
farmaco
fascia
fastoso
fasullo
faticare
fato
favoloso
febbre
fecola
fede
fegato
felpa
feltro
femmina
fendere
fenomeno
fermento
ferro
fertile
fessura
festivo
fetta
feudo
fiaba
fiducia
fifa
figurato
filo
finanza
finestra
finire
fiore
fiscale
fisico
fiume
flacone
flamenco
flebo
flemma
florido
fluente
fluoro
fobico
focaccia
focoso
foderato
foglio
folata
folclore
folgore
fondente
fonetico
fonia
fontana
forbito
forchetta
foresta
formica
fornaio
foro
fortezza
forzare
fosfato
fosso
fracasso
frana
frassino
fratello
freccetta
frenata
fresco
frigo
frollino
fronde
frugale
frutta
fucilata
fucsia
fuggente
fulmine
fulvo
fumante
fumetto
fumoso
fune
funzione
fuoco
furbo
furgone
furore
fuso
futile
gabbiano
gaffe
galateo
gallina
galoppo
gambero
gamma
garanzia
garbo
garofano
garzone
gasdotto
gasolio
gastrico
gatto
gaudio
gazebo
gazzella
geco
gelatina
gelso
gemello
gemmato
gene
genitore
gennaio
genotipo
gergo
ghepardo
ghiaccio
ghisa
giallo
gilda
ginepro
giocare
gioiello
giorno
giove
girato
girone
gittata
giudizio
giurato
giusto
globulo
glutine
gnomo
gobba
golf
gomito
gommone
gonfio
gonna
governo
gracile
grado
grafico
grammo
grande
grattare
gravoso
grazia
greca
gregge
grifone
grigio
grinza
grotta
gruppo
guadagno
guaio
guanto
guardare
gufo
guidare
ibernato
icona
identico
idillio
idolo
idra
idrico
idrogeno
igiene
ignaro
ignorato
ilare
illeso
illogico
illudere
imballo
imbevuto
imbocco
imbuto
immane
immerso
immolato
impacco
impeto
impiego
importo
impronta
inalare
inarcare
inattivo
incanto
incendio
inchino
incisivo
incluso
incontro
incrocio
incubo
indagine
india
indole
inedito
infatti
infilare
inflitto
ingaggio
ingegno
inglese
ingordo
ingrosso
innesco
inodore
inoltrare
inondato
insano
insetto
insieme
insonnia
insulina
intasato
intero
intonaco
intuito
inumidire
invalido
invece
invito
iperbole
ipnotico
ipotesi
ippica
iride
irlanda
ironico
irrigato
irrorare
isolato
isotopo
isterico
istituto
istrice
italia
iterare
labbro
labirinto
lacca
lacerato
lacrima
lacuna
laddove
lago
lampo
lancetta
lanterna
lardoso
larga
laringe
lastra
latenza
latino
lattuga
lavagna
lavoro
legale
leggero
lembo
lentezza
lenza
leone
lepre
lesivo
lessato
lesto
letterale
leva
levigato
libero
lido
lievito
lilla
limatura
limitare
limpido
lineare
lingua
liquido
lira
lirica
lisca
lite
litigio
livrea
locanda
lode
logica
lombare
londra
longevo
loquace
lorenzo
loto
lotteria
luce
lucidato
lumaca
luminoso
lungo
lupo
luppolo
lusinga
lusso
lutto
macabro
macchina
macero
macinato
madama
magico
maglia
magnete
magro
maiolica
malafede
malgrado
malinteso
malsano
malto
malumore
mana
mancia
mandorla
mangiare
manifesto
mannaro
manovra
mansarda
mantide
manubrio
mappa
maratona
marcire
maretta
marmo
marsupio
maschera
massaia
mastino
materasso
matricola
mattone
maturo
mazurca
meandro
meccanico
mecenate
medesimo
meditare
mega
melassa
melis
melodia
meninge
meno
mensola
mercurio
merenda
merlo
meschino
mese
messere
mestolo
metallo
metodo
mettere
miagolare
mica
micelio
michele
microbo
midollo
miele
migliore
milano
milite
mimosa
minerale
mini
minore
mirino
mirtillo
miscela
missiva
misto
misurare
mitezza
mitigare
mitra
mittente
mnemonico
modello
modifica
modulo
mogano
mogio
mole
molosso
monastero
monco
mondina
monetario
monile
monotono
monsone
montato
monviso
mora
mordere
morsicato
mostro
motivato
motosega
motto
movenza
movimento
mozzo
mucca
mucosa
muffa
mughetto
mugnaio
mulatto
mulinello
multiplo
mummia
munto
muovere
murale
musa
muscolo
musica
mutevole
muto
nababbo
nafta
nanometro
narciso
narice
narrato
nascere
nastrare
naturale
nautica
naviglio
nebulosa
necrosi
negativo
negozio
nemmeno
neofita
neretto
nervo
nessuno
nettuno
neutrale
neve
nevrotico
nicchia
ninfa
nitido
nobile
nocivo
nodo
nome
nomina
nordico
normale
norvegese
nostrano
notare
notizia
notturno
novella
nucleo
nulla
numero
nuovo
nutrire
nuvola
nuziale
oasi
obbedire
obbligo
obelisco
oblio
obolo
obsoleto
occasione
occhio
occidente
occorrere
occultare
ocra
oculato
odierno
odorare
offerta
offrire
offuscato
oggetto
oggi
ognuno
olandese
olfatto
oliato
oliva
ologramma
oltre
omaggio
ombelico
ombra
omega
omissione
ondoso
onere
onice
onnivoro
onorevole
onta
operato
opinione
opposto
oracolo
orafo
ordine
orecchino
orefice
orfano
organico
origine
orizzonte
orma
ormeggio
ornativo
orologio
orrendo
orribile
ortensia
ortica
orzata
orzo
osare
oscurare
osmosi
ospedale
ospite
ossa
ossidare
ostacolo
oste
otite
otre
ottagono
ottimo
ottobre
ovale
ovest
ovino
oviparo
ovocito
ovunque
ovviare
ozio
pacchetto
pace
pacifico
padella
padrone
paese
paga
pagina
palazzina
palesare
pallido
palo
palude
pandoro
pannello
paolo
paonazzo
paprica
parabola
parcella
parere
pargolo
pari
parlato
parola
partire
parvenza
parziale
passivo
pasticca
patacca
patologia
pattume
pavone
peccato
pedalare
pedonale
peggio
peloso
penare
pendice
penisola
pennuto
penombra
pensare
pentola
pepe
pepita
perbene
percorso
perdonato
perforare
pergamena
periodo
permesso
perno
perplesso
persuaso
pertugio
pervaso
pesatore
pesista
peso
pestifero
petalo
pettine
petulante
pezzo
piacere
pianta
piattino
piccino
picozza
piega
pietra
piffero
pigiama
pigolio
pigro
pila
pilifero
pillola
pilota
pimpante
pineta
pinna
pinolo
pioggia
piombo
piramide
piretico
pirite
pirolisi
pitone
pizzico
placebo
planare
plasma
platano
plenario
pochezza
poderoso
podismo
poesia
poggiare
polenta
poligono
pollice
polmonite
polpetta
polso
poltrona
polvere
pomice
pomodoro
ponte
popoloso
porfido
poroso
porpora
porre
portata
posa
positivo
possesso
postulato
potassio
potere
pranzo
prassi
pratica
precluso
predica
prefisso
pregiato
prelievo
premere
prenotare
preparato
presenza
pretesto
prevalso
prima
principe
privato
problema
procura
produrre
profumo
progetto
prolunga
promessa
pronome
proposta
proroga
proteso
prova
prudente
prugna
prurito
psiche
pubblico
pudica
pugilato
pugno
pulce
pulito
pulsante
puntare
pupazzo
pupilla
puro
quadro
qualcosa
quasi
querela
quota
raccolto
raddoppio
radicale
radunato
raffica
ragazzo
ragione
ragno
ramarro
ramingo
ramo
randagio
rantolare
rapato
rapina
rappreso
rasatura
raschiato
rasente
rassegna
rastrello
rata
ravveduto
reale
recepire
recinto
recluta
recondito
recupero
reddito
redimere
regalato
registro
regola
regresso
relazione
remare
remoto
renna
replica
reprimere
reputare
resa
residente
responso
restauro
rete
retina
retorica
rettifica
revocato
riassunto
ribadire
ribelle
ribrezzo
ricarica
ricco
ricevere
riciclato
ricordo
ricreduto
ridicolo
ridurre
rifasare
riflesso
riforma
rifugio
rigare
rigettato
righello
rilassato
rilevato
rimanere
rimbalzo
rimedio
rimorchio
rinascita
rincaro
rinforzo
rinnovo
rinomato
rinsavito
rintocco
rinuncia
rinvenire
riparato
ripetuto
ripieno
riportare
ripresa
ripulire
risata
rischio
riserva
risibile
riso
rispetto
ristoro
risultato
risvolto
ritardo
ritegno
ritmico
ritrovo
riunione
riva
riverso
rivincita
rivolto
rizoma
roba
robotico
robusto
roccia
roco
rodaggio
rodere
roditore
rogito
rollio
romantico
rompere
ronzio
rosolare
rospo
rotante
rotondo
rotula
rovescio
rubizzo
rubrica
ruga
rullino
rumine
rumoroso
ruolo
rupe
russare
rustico
sabato
sabbiare
sabotato
sagoma
salasso
saldatura
salgemma
salivare
salmone
salone
saltare
saluto
salvo
sapere
sapido
saporito
saraceno
sarcasmo
sarto
sassoso
satellite
satira
satollo
saturno
savana
savio
saziato
sbadiglio
sbalzo
sbancato
sbarra
sbattere
sbavare
sbendare
sbirciare
sbloccato
sbocciato
sbrinare
sbruffone
sbuffare
scabroso
scadenza
scala
scambiare
scandalo
scapola
scarso
scatenare
scavato
scelto
scenico
scettro
scheda
schiena
sciarpa
scienza
scindere
scippo
sciroppo
scivolo
sclerare
scodella
scolpito
scomparto
sconforto
scoprire
scorta
scossone
scozzese
scriba
scrollare
scrutinio
scuderia
scultore
scuola
scuro
scusare
sdebitare
sdoganare
seccatura
secondo
sedano
seggiola
segnalato
segregato
seguito
selciato
selettivo
sella
selvaggio
semaforo
sembrare
seme
seminato
sempre
senso
sentire
sepolto
sequenza
serata
serbato
sereno
serio
serpente
serraglio
servire
sestina
setola
settimana
sfacelo
sfaldare
sfamato
sfarzoso
sfaticato
sfera
sfida
sfilato
sfinge
sfocato
sfoderare
sfogo
sfoltire
sforzato
sfratto
sfruttato
sfuggito
sfumare
sfuso
sgabello
sgarbato
sgonfiare
sgorbio
sgrassato
sguardo
sibilo
siccome
sierra
sigla
signore
silenzio
sillaba
simbolo
simpatico
simulato
sinfonia
singolo
sinistro
sino
sintesi
sinusoide
sipario
sisma
sistole
situato
slitta
slogatura
sloveno
smarrito
smemorato
smentito
smeraldo
smilzo
smontare
smottato
smussato
snellire
snervato
snodo
sobbalzo
sobrio
soccorso
sociale
sodale
soffitto
sogno
soldato
solenne
solido
sollazzo
solo
solubile
solvente
somatico
somma
sonda
sonetto
sonnifero
sopire
soppeso
sopra
sorgere
sorpasso
sorriso
sorso
sorteggio
sorvolato
sospiro
sosta
sottile
spada
spalla
spargere
spatola
spavento
spazzola
specie
spedire
spegnere
spelatura
speranza
spessore
spettrale
spezzato
spia
spigoloso
spillato
spinoso
spirale
splendido
sportivo
sposo
spranga
sprecare
spronato
spruzzo
spuntino
squillo
sradicare
srotolato
stabile
stacco
staffa
stagnare
stampato
stantio
starnuto
stasera
statuto
stelo
steppa
sterzo
stiletto
stima
stirpe
stivale
stizzoso
stonato
storico
strappo
stregato
stridulo
strozzare
strutto
stuccare
stufo
stupendo
subentro
succoso
sudore
suggerito
sugo
sultano
suonare
superbo
supporto
surgelato
surrogato
sussurro
sutura
svagare
svedese
sveglio
svelare
svenuto
svezia
sviluppo
svista
svizzera
svolta
svuotare
tabacco
tabulato
tacciare
taciturno
tale
talismano
tampone
tannino
tara
tardivo
targato
tariffa
tarpare
tartaruga
tasto
tattico
taverna
tavolata
tazza
teca
tecnico
telefono
temerario
tempo
temuto
tendone
tenero
tensione
tentacolo
teorema
terme
terrazzo
terzetto
tesi
tesserato
testato
tetro
tettoia
tifare
tigella
timbro
tinto
tipico
tipografo
tiraggio
tiro
titanio
titolo
titubante
tizio
tizzone
toccare
tollerare
tolto
tombola
tomo
tonfo
tonsilla
topazio
topologia
toppa
torba
tornare
torrone
tortora
toscano
tossire
tostatura
totano
trabocco
trachea
trafila
tragedia
tralcio
tramonto
transito
trapano
trarre
trasloco
trattato
trave
treccia
tremolio
trespolo
tributo
tricheco
trifoglio
trillo
trincea
trio
tristezza
triturato
trivella
tromba
trono
troppo
trottola
trovare
truccato
tubatura
tuffato
tulipano
tumulto
tunisia
turbare
turchino
tuta
tutela
ubicato
uccello
uccisore
udire
uditivo
uffa
ufficio
uguale
ulisse
ultimato
umano
umile
umorismo
uncinetto
ungere
ungherese
unicorno
unificato
unisono
unitario
unte
uovo
upupa
uragano
urgenza
urlo
usanza
usato
uscito
usignolo
usuraio
utensile
utilizzo
utopia
vacante
vaccinato
vagabondo
vagliato
valanga
valgo
valico
valletta
valoroso
valutare
valvola
vampata
vangare
vanitoso
vano
vantaggio
vanvera
vapore
varano
varcato
variante
vasca
vedetta
vedova
veduto
vegetale
veicolo
velcro
velina
velluto
veloce
venato
vendemmia
vento
verace
verbale
vergogna
verifica
vero
verruca
verticale
vescica
vessillo
vestale
veterano
vetrina
vetusto
viandante
vibrante
vicenda
vichingo
vicinanza
vidimare
vigilia
vigneto
vigore
vile
villano
vimini
vincitore
viola
vipera
virgola
virologo
virulento
viscoso
visione
vispo
vissuto
visura
vita
vitello
vittima
vivanda
vivido
viziare
voce
voga
volatile
volere
volpe
voragine
vulcano
zampogna
zanna
zappato
zattera
zavorra
zefiro
zelante
zelo
zenzero
zerbino
zibetto
zinco
zircone
zitto
zolla
zotico
zucchero
zufolo
zulu
zuppa
"""
words_list = [x for x in s.split("\n") if x is not ""]


#!/usr/bin/python

import re
from wicc.transactions import *
from wicc.serializer import Serializer, Stamper
from wicc.mnemonic import *
from wicc.cryptos.coins.waykicoin import *


class Wallet(object):

    def __init__(self, private_key, main_net=False):
        """
        :param private_key: 私钥
        :param mainnet: 网络参数
        """
        chain_coin = WaykiCoin(testnet=not main_net)
        self.chain_coin = chain_coin
        self.private_key = private_key
        self.public_key = chain_coin.privtopub(private_key)
        self.wallet_address = chain_coin.pubtoaddr(self.public_key)
        self.hash_code = chain_coin.hashcode
        self.version = 1

    @classmethod
    def generate_mnemonic(cls, language=Language.English):
        return random_words(language=language)

    @classmethod
    def private_key_from_mnemonic(cls, mnemonic, mnemonic_language=Language.English, main_net=False):
        coin = WaykiCoin(testnet=not main_net)
        language_mn = to_en_mnemonic_string(mnemonic_language, mnemonic)
        w = coin.wallet(language_mn)
        address = w.new_receiving_address()
        return w.privkey(address)

    @classmethod
    def generate_private_key(cls, main_net=False):
        coin = WaykiCoin(testnet=not main_net)
        w = coin.wallet(random_words())
        return w.privkey(w.new_receiving_address())

    def transfer_tx(self, transfer: TransferTransaction) -> str:
        """
        :param transfer: 转账交易
        :return: 签名后的转账交易
        """
        ser_message = Serializer()\
            .ser_version(self.version)\
            .ser_tx_type(TxType.U_COIN_TRANSFER.value)\
            .ser_valid_height(transfer.valid_height)\
            .ser_regid_or_public_key(transfer.register_id, self.public_key)\
            .ser_fee(int(transfer.fee_amount), transfer.fee_coin_symbol)\
            .ser_transfer(transfer.transfer_list)\
            .ser_memo(transfer.memo)\
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer()\
            .ser_tx_type(TxType.U_COIN_TRANSFER.value)\
            .ser_version(self.version)\
            .ser_valid_height(transfer.valid_height)\
            .ser_regid_or_public_key(transfer.register_id, self.public_key) \
            .ser_fee(int(transfer.fee_amount), transfer.fee_coin_symbol)\
            .ser_transfer(transfer.transfer_list)\
            .ser_memo(transfer.memo)\
            .ser_signature(signature)\
            .to_hex_string()

    # def vote_tx(self, vote: VoteTransaction) -> str:
    #     """
    #     :param vote: 投票交易
    #     :return: 签名后的 投票交易 信息
    #     """
    #     pass
    #
    # def contract_deploy_tx(self, contract_deploy: ContractDeployTransaction) -> str:
    #     """
    #     :param contract_deploy: 合约部署交易
    #     :return: 签名后的 合约部署交易 信息
    #     """

    def contract_call_tx(self, contract_call: ContractCallTransaction) -> str:
        """
        :param contract_call: 合约调用交易
        :return: 签名后的 合约调用交易
        """
        ser_message = Serializer() \
            .ser_version(self.version) \
            .ser_tx_type(TxType.U_CONTRACT_INVOKE.value) \
            .ser_valid_height(contract_call.valid_height) \
            .ser_regid_or_public_key(contract_call.register_id, self.public_key) \
            .ser_regid(contract_call.app_id)\
            .ser_contract_message(contract_call.contract_call_msg)\
            .ser_fee_amount(int(contract_call.fee_amount))\
            .ser_coin_symbol(contract_call.fee_coin_symbol) \
            .ser_amount(int(contract_call.pay_amount), contract_call.pay_coin_symbol)\
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer() \
            .ser_tx_type(TxType.U_CONTRACT_INVOKE.value) \
            .ser_version(self.version) \
            .ser_valid_height(contract_call.valid_height) \
            .ser_regid_or_public_key(contract_call.register_id, self.public_key) \
            .ser_regid(contract_call.app_id)\
            .ser_contract_message(contract_call.contract_call_msg)\
            .ser_fee_amount(int(contract_call.fee_amount)) \
            .ser_coin_symbol(contract_call.fee_coin_symbol) \
            .ser_amount(int(contract_call.pay_amount), contract_call.pay_coin_symbol)\
            .ser_signature(signature)\
            .to_hex_string()

    def dex_limited_price_buy_tx(self, t: DexLimitedPriceBuyTransaction):
        """
        :param t: dex限价买单
        :return: 签名后的 dex限价买单
        """
        ser_message = Serializer()\
            .ser_version(self.version)\
            .ser_tx_type(TxType.DEX_LIMITED_PRICE_BUY_ORDER.value)\
            .ser_valid_height(t.valid_height)\
            .ser_regid_or_public_key(t.register_id, self.public_key)\
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol)\
            .ser_coin_symbol(t.coin_symbol)\
            .ser_coin_symbol(t.asset_symbol)\
            .ser_coin_amount(int(t.asset_amount))\
            .ser_coin_amount(t.price)\
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer() \
            .ser_tx_type(TxType.DEX_LIMITED_PRICE_BUY_ORDER.value) \
            .ser_version(self.version) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_coin_symbol(t.coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_coin_amount(int(t.asset_amount)) \
            .ser_coin_amount(t.price) \
            .ser_signature(signature) \
            .to_hex_string()

    def dex_limited_price_sell_tx(self, t: DexLimitedPriceSellTransaction):
        """
        :param t: dex限价卖单
        :return: 签名后的 dex限价卖单
        """
        ser_message = Serializer() \
            .ser_version(self.version) \
            .ser_tx_type(TxType.DEX_LIMITED_PRICE_SELL_ORDER.value) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_coin_symbol(t.coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_coin_amount(t.asset_amount) \
            .ser_coin_amount(t.price) \
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer() \
            .ser_tx_type(TxType.DEX_LIMITED_PRICE_SELL_ORDER.value) \
            .ser_version(self.version) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_coin_symbol(t.coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_coin_amount(int(t.asset_amount)) \
            .ser_coin_amount(int(t.price)) \
            .ser_signature(signature) \
            .to_hex_string()

    def dex_market_price_buy_tx(self, t: DexMarketPriceBuyTransaction):
        """
        :param t: dex市价买单
        :return: 签名后的 dex市价买单
        """
        ser_message = Serializer() \
            .ser_version(self.version) \
            .ser_tx_type(TxType.DEX_MARKET_PRICE_BUY_ORDER.value) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_coin_symbol(t.coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_coin_amount(int(t.coin_amount)) \
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer() \
            .ser_tx_type(TxType.DEX_MARKET_PRICE_BUY_ORDER.value) \
            .ser_version(self.version) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(t.fee_amount, t.fee_coin_symbol) \
            .ser_coin_symbol(t.coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_coin_amount(int(t.coin_amount)) \
            .ser_signature(signature) \
            .to_hex_string()

    def dex_market_price_sell_tx(self, t: DexMarketPriceSellTransaction):
        """
        :param t: dex市价卖单
        :return: 签名后的 dex市价卖单
        """
        ser_message = Serializer() \
            .ser_version(self.version) \
            .ser_tx_type(TxType.DEX_MARKET_PRICE_SELL_ORDER.value) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_coin_symbol(t.coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_coin_amount(int(t.asset_amount)) \
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer() \
            .ser_tx_type(TxType.DEX_MARKET_PRICE_SELL_ORDER.value) \
            .ser_version(self.version) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_coin_symbol(t.coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_coin_amount(int(t.asset_amount)) \
            .ser_signature(signature) \
            .to_hex_string()

    def dex_cancel_order_tx(self, t: DexCancelOrderTransaction):
        """
        :param t: dex取消订单
        :return: 签名后的 dex取消订单
        """
        ser_message = Serializer() \
            .ser_version(self.version) \
            .ser_tx_type(TxType.DEX_CANCEL_ORDER.value) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_order_id(t.order_id)\
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer()\
            .ser_tx_type(TxType.DEX_CANCEL_ORDER.value) \
            .ser_version(self.version) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_order_id(t.order_id) \
            .ser_signature(signature) \
            .to_hex_string()

    def cdp_stake_tx(self, t: CdpStakeTransaction):
        """
        :param t: 创建/追加cdp交易
        :return: 签名后的 创建/追加cdp交易
        """
        ser_message = Serializer() \
            .ser_version(self.version) \
            .ser_tx_type(TxType.CDP_STAKE.value) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_cdp_id(t.cdp_id)\
            .ser_cdp_stake_list(t.stake_list)\
            .ser_coin_symbol(t.get_coin_symbol)\
            .ser_coin_amount(int(t.get_amount))\
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer()\
            .ser_tx_type(TxType.CDP_STAKE.value)\
            .ser_version(self.version)\
            .ser_valid_height(t.valid_height)\
            .ser_regid_or_public_key(t.register_id, self.public_key)\
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_cdp_id(t.cdp_id) \
            .ser_cdp_stake_list(t.stake_list)\
            .ser_coin_symbol(t.get_coin_symbol)\
            .ser_coin_amount(int(t.get_amount))\
            .ser_signature(signature)\
            .to_hex_string()

    def cdp_redeem_tx(self, t: CdpRedeemTransaction):
        """
        :param t: 赎回cdp交易
        :return: 签名后的 赎回cdp交易
        """
        ser_message = Serializer() \
            .ser_version(self.version) \
            .ser_tx_type(TxType.CDP_REDEEM.value) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_cdp_id(t.cdp_id) \
            .ser_coin_amount(int(t.repay_amount)) \
            .ser_cdp_redeem_list(t.redeem_list) \
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer() \
            .ser_tx_type(TxType.CDP_REDEEM.value) \
            .ser_version(self.version) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_cdp_id(t.cdp_id) \
            .ser_coin_amount(int(t.repay_amount)) \
            .ser_cdp_redeem_list(t.redeem_list) \
            .ser_signature(signature) \
            .to_hex_string()

    def cdp_liquidate_tx(self, t: CdpLiquidateTransaction):
        """
        :param t: 清算cdp交易
        :return: 签名后的 清算cdp交易
        """
        ser_message = Serializer() \
            .ser_version(self.version) \
            .ser_tx_type(TxType.CDP_LIQUIDATE.value) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_cdp_id(t.cdp_id) \
            .ser_coin_symbol(t.liquidate_coin_symbol) \
            .ser_coin_amount(int(t.liquidate_amount))\
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer() \
            .ser_tx_type(TxType.CDP_LIQUIDATE.value) \
            .ser_version(self.version) \
            .ser_valid_height(t.valid_height) \
            .ser_regid_or_public_key(t.register_id, self.public_key) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_cdp_id(t.cdp_id) \
            .ser_coin_symbol(t.liquidate_coin_symbol) \
            .ser_coin_amount(int(t.liquidate_amount)) \
            .ser_signature(signature) \
            .to_hex_string()

    def asset_publish_tx(self, t: AssetPublishTransaction) -> str:
        """
        :param t: 资产发布交易
        :return: 签名后的 资产发布交易
        """
        if not re.match(r"[A-Z]{6,7}$", t.asset_symbol):
            raise Exception("Asset symbol Error")

        renewable = 0
        if t.asset_renewable:
            renewable = 1
        else:
            renewable = 0
        ser_message = Serializer() \
            .ser_version(self.version) \
            .ser_tx_type(TxType.ASSET_ISSUE.value) \
            .ser_valid_height(t.valid_height) \
            .ser_regid(t.register_id) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_regid(t.asset_owner_regid) \
            .ser_coin_symbol(t.asset_name) \
            .ser_coin_amount(renewable) \
            .ser_coin_amount(t.asset_amount) \
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer() \
            .ser_tx_type(TxType.ASSET_ISSUE.value) \
            .ser_version(self.version) \
            .ser_valid_height(t.valid_height) \
            .ser_regid(t.register_id) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_regid(t.asset_owner_regid) \
            .ser_coin_symbol(t.asset_name) \
            .ser_coin_amount(renewable) \
            .ser_coin_amount(t.asset_amount) \
            .ser_signature(signature) \
            .to_hex_string()

    def asset_update_tx(self, t: AssetUpdateTransaction) -> str:
        """
        :param t: 资产发布交易
        :return: 签名后的 资产发布交易
        """
        if not re.match(r"[A-Z]{6,7}$", t.asset_symbol):
            raise Exception("Asset symbol Error")

        ser_message = Serializer() \
            .ser_version(self.version) \
            .ser_tx_type(TxType.ASSET_UPDATE.value) \
            .ser_valid_height(t.valid_height) \
            .ser_regid(t.register_id) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_asset_type_and_value(t.asset_update_type, t.asset_update_value) \
            .to_message()

        signature = Stamper.stamp(ser_message, self.private_key)

        return Serializer() \
            .ser_tx_type(TxType.ASSET_UPDATE.value) \
            .ser_version(self.version) \
            .ser_valid_height(t.valid_height) \
            .ser_regid(t.register_id) \
            .ser_fee(int(t.fee_amount), t.fee_coin_symbol) \
            .ser_coin_symbol(t.asset_symbol) \
            .ser_asset_type_and_value(t.asset_update_type, t.asset_update_value) \
            .ser_signature(signature) \
            .to_hex_string()



from cryptos.electrumx_client import rpc_send_and_wait, rpc_send

host = "electrum.akinbo.org"
port = 50001
method = "blockchain.address.listunspent"
params = ["1AoZAu5Kif5MTkaNWbHiXGdcNEgFGVqDgB"]

rpc_send(host, port, method, params)
print('yes')
import time
time.sleep(2)
#print(result)


from .main import *


def serialize_header(inp):
    o = encode(inp['version'], 256, 4)[::-1] + \
        inp['prevhash'].decode('hex')[::-1] + \
        inp['merkle_root'].decode('hex')[::-1] + \
        encode(inp['timestamp'], 256, 4)[::-1] + \
        encode(inp['bits'], 256, 4)[::-1] + \
        encode(inp['nonce'], 256, 4)[::-1]
    h = bin_sha256(bin_sha256(o))[::-1].encode('hex')
    assert h == inp['hash'], (sha256(o), inp['hash'])
    return o.encode('hex')


def deserialize_header(inp):
    inp = inp.decode('hex')
    return {
        "version": decode(inp[:4][::-1], 256),
        "prevhash": inp[4:36][::-1].encode('hex'),
        "merkle_root": inp[36:68][::-1].encode('hex'),
        "timestamp": decode(inp[68:72][::-1], 256),
        "bits": decode(inp[72:76][::-1], 256),
        "nonce": decode(inp[76:80][::-1], 256),
        "hash": bin_sha256(bin_sha256(inp))[::-1].encode('hex')
    }


def mk_merkle_proof(header, hashes, index):
    nodes = [safe_from_hex(h)[::-1] for h in hashes]
    if len(nodes) % 2 and len(nodes) > 2:
        nodes.append(nodes[-1])
    layers = [nodes]
    while len(nodes) > 1:
        newnodes = []
        for i in range(0, len(nodes) - 1, 2):
            newnodes.append(bin_sha256(bin_sha256(nodes[i] + nodes[i+1])))
        if len(newnodes) % 2 and len(newnodes) > 2:
            newnodes.append(newnodes[-1])
        nodes = newnodes
        layers.append(nodes)
    # Sanity check, make sure merkle root is valid
    assert bytes_to_hex_string(nodes[0][::-1]) == header['merkle_root']
    merkle_siblings = \
        [layers[i][(index >> i) ^ 1] for i in range(len(layers)-1)]
    return {
        "hash": hashes[index],
        "siblings": [bytes_to_hex_string(x[::-1]) for x in merkle_siblings],
        "header": header
    }



from . import base_insight as insight

def get_url(coin_symbol):
    if coin_symbol == "btg":
        return "https://btgexplorer.com/api"
    raise NotImplementedError("Explorer unavailable for Bitcoin Gold testnet")

def unspent(*args, coin_symbol="btg"):
    base_url = get_url(coin_symbol)
    return insight.unspent(base_url, *args)

def fetchtx(txhash, coin_symbol="btg"):
    base_url = get_url(coin_symbol="btg")
    return insight.fetchtx(base_url, txhash)

def txinputs(txhash, coin_symbol="btg"):
    base_url = get_url(coin_symbol)
    return insight.txinputs(base_url, txhash)

def pushtx(tx, coin_symbol="btg"):
    base_url = get_url(coin_symbol)
    return insight.pushtx(base_url, coin_symbol.upper(), tx)

def history(*args,  coin_symbol="btg"):
    base_url = get_url(coin_symbol)
    return insight.history(base_url, *args)

def block_height(tx, coin_symbol="btg"):
    base_url = get_url(coin_symbol)
    return insight.block_height(base_url, tx)

def current_block_height(coin_symbol="btg"):
    base_url = get_url(coin_symbol)
    return insight.current_block_height(base_url)

def block_info(height, coin_symbol="btg"):
    base_url = get_url(coin_symbol)
    return insight.block_info(base_url, height)


# coding=utf-8
from wicc.wiccrpc.box.RpcManager import RpcManager


class Block(object):

    @classmethod
    def current_height(cls):
        """
        获取当前节点区块高度
        :return: 高度值
        """
        data = {
            "method": "getblockcount"
        }
        return RpcManager.request(data)['result']

    @classmethod
    def block_hash(cls, height):
        """
        查询区块哈希值
        :param height: 区块高度
        :return: 区块哈希
        """
        data = {
            "method": "getblockhash",
            "params": [height]
        }
        return RpcManager.request(data)['result']['hash']

    @classmethod
    def block_info(cls, block):
        """
        获取区块信息
        :param block: 区块高度/区块哈希
        :return: 区块信息
        """
        data = {
            "method": "getblock",
            "params": [block]
        }
        return RpcManager.request(data)['result']




# coding=utf-8
from wicc.wiccrpc.box.RpcManager import RpcManager


class Wallet(object):
    """
    钱包即账号的集合，可以管理多个账号
    """

    @classmethod
    def encrypt_wallet(cls, password):
        """
        对钱包进行设置密码
        :return: 加密状态（true:加密成功，false:加密失败）
        """
        data = {
            "method": "encryptwallet",
            "params": [password]
        }
        return RpcManager.request(data)["result"]['encrypt']

    @classmethod
    def unlock_wallet(cls, password, second=3600):
        """
        解锁钱包，并设置解锁持续的时间
        :param password: 钱包密码
        :param second: 解锁持续的时间（秒），默认3600秒
        :return: 解锁状态（true:解锁成功、false:解锁失败）
        """
        data = {
            "method": "walletpassphrase",
            "params": [password, second]
        }
        return RpcManager.request(data)["result"]["passphrase"]

    @classmethod
    def lock_wallet(cls):
        """
        锁定钱包
        :return: rue:锁定成功, false:锁定失败
        """
        data = {
            "method": "walletlock",
            "params": []
        }
        return RpcManager.request(data)['result']['walletlock']

    @classmethod
    def update_wallet_password(cls, original_password, new_password):
        """
        更新钱包的加密密码
        :param original_password: 原密码
        :param new_password: 新密码
        :return: 更新状态 （true:修改成功、false:修改失败）
        """
        data = {
            "method": "walletpassphrasechange",
            "parames": [original_password, new_password]
        }
        return RpcManager.request(data)["result"]["chgpwd"]

    @classmethod
    def export_wallet(cls, path="./"):
        """
        导出钱包
        :param path: 到处的路径
        :return: （info 导出钱包的结果信息，key size：导出钱包中包含的地址账号数量）
        """
        data = {
            "method": "dumpwallet",
            "parames": [path]
        }
        result = RpcManager.request(data)["result"]
        return result['info'], result['key size']

    @classmethod
    def import_wallet(cls, path):
        """
        导入钱包
        :param path: 导入的路径
        :return: 导入的账号数量
        """
        data = {
            "method": "importwallet",
            "parames": [path]
        }
        return RpcManager.request(data)["result"]["imorpt key size"]

    @classmethod
    def backup_wallet(cls, path="./"):
        """
        备份钱包, 即备份wallet.dat文件
        :param path: 备份的路径
        :return:
        """
        data = {
            "method": "backupwallet",
            "parames": [path]
        }
        return RpcManager.request(data)["result"]

    @classmethod
    def wallet_info(cls):
        """
        获取钱包信息
        :return: 钱包信息
        """
        data = {
            "method": "getwalletinfo"
        }
        return RpcManager.request(data)['result']




#!/usr/bin/python
# -*- coding: utf-8 -*-


import requests
import json
import demjson


class BaasManager(object):

    # 默认测试网
    __is_main_net = False
    __main_net_baas_url = "https://baas.wiccdev.org/v2/api"
    __test_net_baas_url = "https://baas-test.wiccdev.org/v2/api"

    @classmethod
    def set_net(cls, is_main_net=False):
        if isinstance(is_main_net, bool):
            cls.__is_main_net = is_main_net
        return cls

    @classmethod
    def __post(cls, uri, request):
        if cls.__is_main_net:
            url = cls.__main_net_baas_url + uri
        else:
            url = cls.__test_net_baas_url + uri
        header = {
            "Content-Type": "application/json"
        }
        request = json.dumps(request)
        resp = requests.post(url=url, data=request, headers=header)
        return demjson.decode(resp.text)

    @classmethod
    def get_valid_height(cls):
        uri = "/block/getblockcount"
        return cls.__post(uri, {})["data"]

    @classmethod
    def submit_tx(cls, tx):
        uri = "/transaction/sendrawtx"
        request = {
            "rawtx": tx
        }
        return cls.__post(uri, request)

    @classmethod
    def decode_raw_tx(cls):
        pass

    @classmethod
    def transaction_detail(cls):
        pass


    @classmethod
    def cdp_info(cls, id):
        pass

    @classmethod
    def stable_coin_info(cls):
        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
s = """
abandon
ability
able
about
above
absent
absorb
abstract
absurd
abuse
access
accident
account
accuse
achieve
acid
acoustic
acquire
across
act
action
actor
actress
actual
adapt
add
addict
address
adjust
admit
adult
advance
advice
aerobic
affair
afford
afraid
again
age
agent
agree
ahead
aim
air
airport
aisle
alarm
album
alcohol
alert
alien
all
alley
allow
almost
alone
alpha
already
also
alter
always
amateur
amazing
among
amount
amused
analyst
anchor
ancient
anger
angle
angry
animal
ankle
announce
annual
another
answer
antenna
antique
anxiety
any
apart
apology
appear
apple
approve
april
arch
arctic
area
arena
argue
arm
armed
armor
army
around
arrange
arrest
arrive
arrow
art
artefact
artist
artwork
ask
aspect
assault
asset
assist
assume
asthma
athlete
atom
attack
attend
attitude
attract
auction
audit
august
aunt
author
auto
autumn
average
avocado
avoid
awake
aware
away
awesome
awful
awkward
axis
baby
bachelor
bacon
badge
bag
balance
balcony
ball
bamboo
banana
banner
bar
barely
bargain
barrel
base
basic
basket
battle
beach
bean
beauty
because
become
beef
before
begin
behave
behind
believe
below
belt
bench
benefit
best
betray
better
between
beyond
bicycle
bid
bike
bind
biology
bird
birth
bitter
black
blade
blame
blanket
blast
bleak
bless
blind
blood
blossom
blouse
blue
blur
blush
board
boat
body
boil
bomb
bone
bonus
book
boost
border
boring
borrow
boss
bottom
bounce
box
boy
bracket
brain
brand
brass
brave
bread
breeze
brick
bridge
brief
bright
bring
brisk
broccoli
broken
bronze
broom
brother
brown
brush
bubble
buddy
budget
buffalo
build
bulb
bulk
bullet
bundle
bunker
burden
burger
burst
bus
business
busy
butter
buyer
buzz
cabbage
cabin
cable
cactus
cage
cake
call
calm
camera
camp
can
canal
cancel
candy
cannon
canoe
canvas
canyon
capable
capital
captain
car
carbon
card
cargo
carpet
carry
cart
case
cash
casino
castle
casual
cat
catalog
catch
category
cattle
caught
cause
caution
cave
ceiling
celery
cement
census
century
cereal
certain
chair
chalk
champion
change
chaos
chapter
charge
chase
chat
cheap
check
cheese
chef
cherry
chest
chicken
chief
child
chimney
choice
choose
chronic
chuckle
chunk
churn
cigar
cinnamon
circle
citizen
city
civil
claim
clap
clarify
claw
clay
clean
clerk
clever
click
client
cliff
climb
clinic
clip
clock
clog
close
cloth
cloud
clown
club
clump
cluster
clutch
coach
coast
coconut
code
coffee
coil
coin
collect
color
column
combine
come
comfort
comic
common
company
concert
conduct
confirm
congress
connect
consider
control
convince
cook
cool
copper
copy
coral
core
corn
correct
cost
cotton
couch
country
couple
course
cousin
cover
coyote
crack
cradle
craft
cram
crane
crash
crater
crawl
crazy
cream
credit
creek
crew
cricket
crime
crisp
critic
crop
cross
crouch
crowd
crucial
cruel
cruise
crumble
crunch
crush
cry
crystal
cube
culture
cup
cupboard
curious
current
curtain
curve
cushion
custom
cute
cycle
dad
damage
damp
dance
danger
daring
dash
daughter
dawn
day
deal
debate
debris
decade
december
decide
decline
decorate
decrease
deer
defense
define
defy
degree
delay
deliver
demand
demise
denial
dentist
deny
depart
depend
deposit
depth
deputy
derive
describe
desert
design
desk
despair
destroy
detail
detect
develop
device
devote
diagram
dial
diamond
diary
dice
diesel
diet
differ
digital
dignity
dilemma
dinner
dinosaur
direct
dirt
disagree
discover
disease
dish
dismiss
disorder
display
distance
divert
divide
divorce
dizzy
doctor
document
dog
doll
dolphin
domain
donate
donkey
donor
door
dose
double
dove
draft
dragon
drama
drastic
draw
dream
dress
drift
drill
drink
drip
drive
drop
drum
dry
duck
dumb
dune
during
dust
dutch
duty
dwarf
dynamic
eager
eagle
early
earn
earth
easily
east
easy
echo
ecology
economy
edge
edit
educate
effort
egg
eight
either
elbow
elder
electric
elegant
element
elephant
elevator
elite
else
embark
embody
embrace
emerge
emotion
employ
empower
empty
enable
enact
end
endless
endorse
enemy
energy
enforce
engage
engine
enhance
enjoy
enlist
enough
enrich
enroll
ensure
enter
entire
entry
envelope
episode
equal
equip
era
erase
erode
erosion
error
erupt
escape
essay
essence
estate
eternal
ethics
evidence
evil
evoke
evolve
exact
example
excess
exchange
excite
exclude
excuse
execute
exercise
exhaust
exhibit
exile
exist
exit
exotic
expand
expect
expire
explain
expose
express
extend
extra
eye
eyebrow
fabric
face
faculty
fade
faint
faith
fall
false
fame
family
famous
fan
fancy
fantasy
farm
fashion
fat
fatal
father
fatigue
fault
favorite
feature
february
federal
fee
feed
feel
female
fence
festival
fetch
fever
few
fiber
fiction
field
figure
file
film
filter
final
find
fine
finger
finish
fire
firm
first
fiscal
fish
fit
fitness
fix
flag
flame
flash
flat
flavor
flee
flight
flip
float
flock
floor
flower
fluid
flush
fly
foam
focus
fog
foil
fold
follow
food
foot
force
forest
forget
fork
fortune
forum
forward
fossil
foster
found
fox
fragile
frame
frequent
fresh
friend
fringe
frog
front
frost
frown
frozen
fruit
fuel
fun
funny
furnace
fury
future
gadget
gain
galaxy
gallery
game
gap
garage
garbage
garden
garlic
garment
gas
gasp
gate
gather
gauge
gaze
general
genius
genre
gentle
genuine
gesture
ghost
giant
gift
giggle
ginger
giraffe
girl
give
glad
glance
glare
glass
glide
glimpse
globe
gloom
glory
glove
glow
glue
goat
goddess
gold
good
goose
gorilla
gospel
gossip
govern
gown
grab
grace
grain
grant
grape
grass
gravity
great
green
grid
grief
grit
grocery
group
grow
grunt
guard
guess
guide
guilt
guitar
gun
gym
habit
hair
half
hammer
hamster
hand
happy
harbor
hard
harsh
harvest
hat
have
hawk
hazard
head
health
heart
heavy
hedgehog
height
hello
helmet
help
hen
hero
hidden
high
hill
hint
hip
hire
history
hobby
hockey
hold
hole
holiday
hollow
home
honey
hood
hope
horn
horror
horse
hospital
host
hotel
hour
hover
hub
huge
human
humble
humor
hundred
hungry
hunt
hurdle
hurry
hurt
husband
hybrid
ice
icon
idea
identify
idle
ignore
ill
illegal
illness
image
imitate
immense
immune
impact
impose
improve
impulse
inch
include
income
increase
index
indicate
indoor
industry
infant
inflict
inform
inhale
inherit
initial
inject
injury
inmate
inner
innocent
input
inquiry
insane
insect
inside
inspire
install
intact
interest
into
invest
invite
involve
iron
island
isolate
issue
item
ivory
jacket
jaguar
jar
jazz
jealous
jeans
jelly
jewel
job
join
joke
journey
joy
judge
juice
jump
jungle
junior
junk
just
kangaroo
keen
keep
ketchup
key
kick
kid
kidney
kind
kingdom
kiss
kit
kitchen
kite
kitten
kiwi
knee
knife
knock
know
lab
label
labor
ladder
lady
lake
lamp
language
laptop
large
later
latin
laugh
laundry
lava
law
lawn
lawsuit
layer
lazy
leader
leaf
learn
leave
lecture
left
leg
legal
legend
leisure
lemon
lend
length
lens
leopard
lesson
letter
level
liar
liberty
library
license
life
lift
light
like
limb
limit
link
lion
liquid
list
little
live
lizard
load
loan
lobster
local
lock
logic
lonely
long
loop
lottery
loud
lounge
love
loyal
lucky
luggage
lumber
lunar
lunch
luxury
lyrics
machine
mad
magic
magnet
maid
mail
main
major
make
mammal
man
manage
mandate
mango
mansion
manual
maple
marble
march
margin
marine
market
marriage
mask
mass
master
match
material
math
matrix
matter
maximum
maze
meadow
mean
measure
meat
mechanic
medal
media
melody
melt
member
memory
mention
menu
mercy
merge
merit
merry
mesh
message
metal
method
middle
midnight
milk
million
mimic
mind
minimum
minor
minute
miracle
mirror
misery
miss
mistake
mix
mixed
mixture
mobile
model
modify
mom
moment
monitor
monkey
monster
month
moon
moral
more
morning
mosquito
mother
motion
motor
mountain
mouse
move
movie
much
muffin
mule
multiply
muscle
museum
mushroom
music
must
mutual
myself
mystery
myth
naive
name
napkin
narrow
nasty
nation
nature
near
neck
need
negative
neglect
neither
nephew
nerve
nest
net
network
neutral
never
news
next
nice
night
noble
noise
nominee
noodle
normal
north
nose
notable
note
nothing
notice
novel
now
nuclear
number
nurse
nut
oak
obey
object
oblige
obscure
observe
obtain
obvious
occur
ocean
october
odor
off
offer
office
often
oil
okay
old
olive
olympic
omit
once
one
onion
online
only
open
opera
opinion
oppose
option
orange
orbit
orchard
order
ordinary
organ
orient
original
orphan
ostrich
other
outdoor
outer
output
outside
oval
oven
over
own
owner
oxygen
oyster
ozone
pact
paddle
page
pair
palace
palm
panda
panel
panic
panther
paper
parade
parent
park
parrot
party
pass
patch
path
patient
patrol
pattern
pause
pave
payment
peace
peanut
pear
peasant
pelican
pen
penalty
pencil
people
pepper
perfect
permit
person
pet
phone
photo
phrase
physical
piano
picnic
picture
piece
pig
pigeon
pill
pilot
pink
pioneer
pipe
pistol
pitch
pizza
place
planet
plastic
plate
play
please
pledge
pluck
plug
plunge
poem
poet
point
polar
pole
police
pond
pony
pool
popular
portion
position
possible
post
potato
pottery
poverty
powder
power
practice
praise
predict
prefer
prepare
present
pretty
prevent
price
pride
primary
print
priority
prison
private
prize
problem
process
produce
profit
program
project
promote
proof
property
prosper
protect
proud
provide
public
pudding
pull
pulp
pulse
pumpkin
punch
pupil
puppy
purchase
purity
purpose
purse
push
put
puzzle
pyramid
quality
quantum
quarter
question
quick
quit
quiz
quote
rabbit
raccoon
race
rack
radar
radio
rail
rain
raise
rally
ramp
ranch
random
range
rapid
rare
rate
rather
raven
raw
razor
ready
real
reason
rebel
rebuild
recall
receive
recipe
record
recycle
reduce
reflect
reform
refuse
region
regret
regular
reject
relax
release
relief
rely
remain
remember
remind
remove
render
renew
rent
reopen
repair
repeat
replace
report
require
rescue
resemble
resist
resource
response
result
retire
retreat
return
reunion
reveal
review
reward
rhythm
rib
ribbon
rice
rich
ride
ridge
rifle
right
rigid
ring
riot
ripple
risk
ritual
rival
river
road
roast
robot
robust
rocket
romance
roof
rookie
room
rose
rotate
rough
round
route
royal
rubber
rude
rug
rule
run
runway
rural
sad
saddle
sadness
safe
sail
salad
salmon
salon
salt
salute
same
sample
sand
satisfy
satoshi
sauce
sausage
save
say
scale
scan
scare
scatter
scene
scheme
school
science
scissors
scorpion
scout
scrap
screen
script
scrub
sea
search
season
seat
second
secret
section
security
seed
seek
segment
select
sell
seminar
senior
sense
sentence
series
service
session
settle
setup
seven
shadow
shaft
shallow
share
shed
shell
sheriff
shield
shift
shine
ship
shiver
shock
shoe
shoot
shop
short
shoulder
shove
shrimp
shrug
shuffle
shy
sibling
sick
side
siege
sight
sign
silent
silk
silly
silver
similar
simple
since
sing
siren
sister
situate
six
size
skate
sketch
ski
skill
skin
skirt
skull
slab
slam
sleep
slender
slice
slide
slight
slim
slogan
slot
slow
slush
small
smart
smile
smoke
smooth
snack
snake
snap
sniff
snow
soap
soccer
social
sock
soda
soft
solar
soldier
solid
solution
solve
someone
song
soon
sorry
sort
soul
sound
soup
source
south
space
spare
spatial
spawn
speak
special
speed
spell
spend
sphere
spice
spider
spike
spin
spirit
split
spoil
sponsor
spoon
sport
spot
spray
spread
spring
spy
square
squeeze
squirrel
stable
stadium
staff
stage
stairs
stamp
stand
start
state
stay
steak
steel
stem
step
stereo
stick
still
sting
stock
stomach
stone
stool
story
stove
strategy
street
strike
strong
struggle
student
stuff
stumble
style
subject
submit
subway
success
such
sudden
suffer
sugar
suggest
suit
summer
sun
sunny
sunset
super
supply
supreme
sure
surface
surge
surprise
surround
survey
suspect
sustain
swallow
swamp
swap
swarm
swear
sweet
swift
swim
swing
switch
sword
symbol
symptom
syrup
system
table
tackle
tag
tail
talent
talk
tank
tape
target
task
taste
tattoo
taxi
teach
team
tell
ten
tenant
tennis
tent
term
test
text
thank
that
theme
then
theory
there
they
thing
this
thought
three
thrive
throw
thumb
thunder
ticket
tide
tiger
tilt
timber
time
tiny
tip
tired
tissue
title
toast
tobacco
today
toddler
toe
together
toilet
token
tomato
tomorrow
tone
tongue
tonight
tool
tooth
top
topic
topple
torch
tornado
tortoise
toss
total
tourist
toward
tower
town
toy
track
trade
traffic
tragic
train
transfer
trap
trash
travel
tray
treat
tree
trend
trial
tribe
trick
trigger
trim
trip
trophy
trouble
truck
true
truly
trumpet
trust
truth
try
tube
tuition
tumble
tuna
tunnel
turkey
turn
turtle
twelve
twenty
twice
twin
twist
two
type
typical
ugly
umbrella
unable
unaware
uncle
uncover
under
undo
unfair
unfold
unhappy
uniform
unique
unit
universe
unknown
unlock
until
unusual
unveil
update
upgrade
uphold
upon
upper
upset
urban
urge
usage
use
used
useful
useless
usual
utility
vacant
vacuum
vague
valid
valley
valve
van
vanish
vapor
various
vast
vault
vehicle
velvet
vendor
venture
venue
verb
verify
version
very
vessel
veteran
viable
vibrant
vicious
victory
video
view
village
vintage
violin
virtual
virus
visa
visit
visual
vital
vivid
vocal
voice
void
volcano
volume
vote
voyage
wage
wagon
wait
walk
wall
walnut
want
warfare
warm
warrior
wash
wasp
waste
water
wave
way
wealth
weapon
wear
weasel
weather
web
wedding
weekend
weird
welcome
west
wet
whale
what
wheat
wheel
when
where
whip
whisper
wide
width
wife
wild
will
win
window
wine
wing
wink
winner
winter
wire
wisdom
wise
wish
witness
wolf
woman
wonder
wood
wool
word
work
world
worry
worth
wrap
wreck
wrestle
wrist
write
wrong
yard
year
yellow
you
young
youth
zebra
zero
zone
zoo
"""

words_list = [x for x in s.split("\n") if x is not ""]


from .base import BaseCoin
from ..transaction import SIGHASH_ALL, SIGHASH_FORKID
from ..explorers import blockdozer

class BitcoinCash(BaseCoin):
    coin_symbol = "bch"
    display_name = "Bitcoin Cash"
    segwit_supported = False
    magicbyte = 0
    script_magicbyte = 5
    wif_prefix = 0x80
    hd_path = 145
    explorer = blockdozer
    hashcode = SIGHASH_ALL | SIGHASH_FORKID
    testnet_overrides = {
        'display_name': "Bitcoin Cash Testnet",
        'coin_symbol': "tbch",
        'magicbyte': 111,
        'script_magicbyte': 196,
        'wif_prefix': 0xef,
        'xprv_headers': {
            'p2pkh': 0x04358394,
        },
        'xpub_headers': {
            'p2pkh': 0x043587cf,
        },
        'hd_path': 1,
    }

    def __init__(self, legacy=False, testnet=False, **kwargs):
        super(BitcoinCash, self).__init__(testnet=testnet, **kwargs)
        self.hd_path = 0 if legacy and testnet else self.hd_path



from .blocks import *
from .composite import *
from .deterministic import *
from .main import *
from .mnemonic import *
from .py2specials import *
from .py3specials import *
from .stealth import *
from .transaction import *
from .coins import *
from .keystore import *
from .wallet import *



from . import base_insight as insight

base_url = "https://%s.blockdozer.com/insight-api"

def unspent(*args, coin_symbol="bcc"):
    base_url_for_coin = base_url % coin_symbol
    return insight.unspent(base_url_for_coin, *args)

def pushtx(tx,  coin_symbol="bcc"):
    base_url_for_coin = base_url % coin_symbol
    return insight.pushtx(base_url_for_coin, coin_symbol, tx)

def fetchtx(txhash, coin_symbol="bcc"):
    base_url_for_coin = base_url % coin_symbol
    return insight.fetchtx(base_url_for_coin, txhash)

def txinputs(txhash, coin_symbol="bcc"):
    base_url_for_coin = base_url % coin_symbol
    return insight.txinputs(base_url_for_coin, txhash)

def history(*args,  coin_symbol="bcc"):
    base_url_for_coin = base_url % coin_symbol
    return insight.history(base_url_for_coin, *args)

def block_height(tx, coin_symbol="bcc"):
    base_url_for_coin = base_url % coin_symbol
    return insight.block_height(base_url_for_coin, tx)

def current_block_height(coin_symbol="bcc"):
    base_url_for_coin = base_url % coin_symbol
    return insight.current_block_height(base_url_for_coin)

def block_info(height, coin_symbol="bcc"):
    base_url_for_coin = base_url % coin_symbol
    return insight.block_info(base_url_for_coin, height)


#!/usr/bin/python
import binascii, re, copy
from .main import *
from _functools import reduce

### Hex to bin converter and vice versa for objects


def json_is_base(obj, base):
    if not is_python2 and isinstance(obj, bytes):
        return False
    
    alpha = get_code_string(base)
    if isinstance(obj, string_types):
        for i in range(len(obj)):
            if alpha.find(obj[i]) == -1:
                return False
        return True
    elif isinstance(obj, int_types) or obj is None:
        return True
    elif isinstance(obj, list):
        for i in range(len(obj)):
            if not json_is_base(obj[i], base):
                return False
        return True
    else:
        for x in obj:
            if not json_is_base(obj[x], base):
                return False
        return True

def json_changebase(obj, changer):
    if isinstance(obj, string_or_bytes_types):
        return changer(obj)
    elif isinstance(obj, int_types) or obj is None:
        return obj
    elif isinstance(obj, list):
        return [json_changebase(x, changer) for x in obj]
    return dict((x, json_changebase(obj[x], changer)) for x in obj)

# Hashing transactions for signing

SIGHASH_ALL = 1
SIGHASH_NONE = 2
SIGHASH_SINGLE = 3
# this works like SIGHASH_ANYONECANPAY | SIGHASH_ALL, might as well make it explicit while
# we fix the constant
SIGHASH_ANYONECANPAY = 0x81
SIGHASH_FORKID = 0x40

def encode_1_byte(val):
    return encode(val, 256, 1)[::-1]

def encode_4_bytes(val):
    return encode(val, 256, 4)[::-1]

def encode_8_bytes(val):
    return encode(val, 256, 8)[::-1]

def list_to_bytes(vals):
    return ''.join(vals) if is_python2 else reduce(lambda x, y: x + y, vals, bytes())

def dbl_sha256_list(vals):
    return bin_dbl_sha256(list_to_bytes(vals))


# Transaction serialization and deserialization

def is_segwit(tx):
    return tx[4] == 0

def deserialize(tx):
    if isinstance(tx, str) and re.match('^[0-9a-fA-F]*$', tx):
        # tx = bytes(bytearray.fromhex(tx))
        return json_changebase(deserialize(binascii.unhexlify(tx)),
                               lambda x: safe_hexlify(x))
    # http://stackoverflow.com/questions/4851463/python-closure-write-to-variable-in-parent-scope
    # Python's scoping rules are demented, requiring me to make pos an object
    # so that it is call-by-reference
    pos = [0]

    def read_as_int(bytez):
        pos[0] += bytez
        return decode(tx[pos[0] - bytez:pos[0]][::-1], 256)

    def read_var_int():
        pos[0] += 1
        val = from_byte_to_int(tx[pos[0] - 1])
        if val < 253:
            return val
        return read_as_int(pow(2, val - 252))

    def read_bytes(bytez):
        pos[0] += bytez
        return tx[pos[0] - bytez:pos[0]]

    def read_var_string():
        size = read_var_int()
        return read_bytes(size)

    def read_segwit_string():
        size = read_var_int()
        return num_to_var_int(size)+read_bytes(size)

    obj = {"ins": [], "outs": []}
    obj["version"] = read_as_int(4)
    has_witness = is_segwit(tx)
    if has_witness:
        obj['marker'] = read_as_int(1)
        obj['flag'] = read_as_int(1)
    ins = read_var_int()
    for i in range(ins):
        obj["ins"].append({
            "outpoint": {
                "hash": read_bytes(32)[::-1],
                "index": read_as_int(4)
            },
            "script": read_var_string(),
            "sequence": read_as_int(4)
        })
    outs = read_var_int()
    for i in range(outs):
        obj["outs"].append({
            "value": read_as_int(8),
            "script": read_var_string()
        })
    if has_witness:
        obj['witness'] = []
        for i in range(ins):
            number = read_var_int()
            scriptCode = []
            for i in range(number):
                scriptCode.append(read_segwit_string())
            obj['witness'].append({
                'number': number,
                'scriptCode': list_to_bytes(scriptCode)
            })
    obj["locktime"] = read_as_int(4)
    return obj

def serialize(txobj, include_witness=True):
    if isinstance(txobj, bytes):
        txobj = bytes_to_hex_string(txobj)
    o = []
    if json_is_base(txobj, 16):
        json_changedbase = json_changebase(txobj, lambda x: binascii.unhexlify(x))
        hexlified = safe_hexlify(serialize(json_changedbase, include_witness=include_witness))
        return hexlified
    o.append(encode_4_bytes(txobj["version"]))
    if include_witness and all(k in txobj.keys() for k in ['marker', 'flag']):
        o.append(encode_1_byte(txobj["marker"]))
        o.append(encode_1_byte(txobj["flag"]))
    o.append(num_to_var_int(len(txobj["ins"])))
    for inp in txobj["ins"]:
        o.append(inp["outpoint"]["hash"][::-1])
        o.append(encode_4_bytes(inp["outpoint"]["index"]))
        o.append(num_to_var_int(len(inp["script"])) + (inp["script"] if inp["script"] or is_python2 else bytes()))
        o.append(encode_4_bytes(inp["sequence"]))
    o.append(num_to_var_int(len(txobj["outs"])))

    for out in txobj["outs"]:
        o.append(encode_8_bytes(out["value"]))
        o.append(num_to_var_int(len(out["script"])) + out["script"])
    if include_witness and "witness" in txobj.keys():
        for witness in txobj["witness"]:
            o.append(num_to_var_int(witness["number"]) + (witness["scriptCode"] if witness["scriptCode"] or is_python2 else bytes()))
    o.append(encode_4_bytes(txobj["locktime"]))
    return list_to_bytes(o)

# https://github.com/Bitcoin-UAHF/spec/blob/master/replay-protected-sighash.md#OP_CHECKSIG
def uahf_digest(txobj, i):
    if isinstance(txobj, bytes):
        txobj = bytes_to_hex_string(txobj)
    o = []

    if json_is_base(txobj, 16):
        txobj = json_changebase(txobj, lambda x: binascii.unhexlify(x))
    o.append(encode(txobj["version"], 256, 4)[::-1])

    serialized_ins = []
    for inp in txobj["ins"]:
        serialized_ins.append(inp["outpoint"]["hash"][::-1])
        serialized_ins.append(encode_4_bytes(inp["outpoint"]["index"]))
    inputs_hashed = dbl_sha256_list(serialized_ins)
    o.append(inputs_hashed)

    sequences = dbl_sha256_list([encode_4_bytes(inp["sequence"]) for inp in txobj['ins']])
    o.append(sequences)

    inp = txobj['ins'][i]
    o.append(inp["outpoint"]["hash"][::-1])
    o.append(encode_4_bytes(inp["outpoint"]["index"]))
    o.append(num_to_var_int(len(inp["script"])) + (inp["script"] if inp["script"] or is_python2 else bytes()))
    o.append(encode_8_bytes(inp['amount']))
    o.append(encode_4_bytes(inp['sequence']))

    serialized_outs = []
    for out in txobj["outs"]:
        serialized_outs.append(encode_8_bytes(out["value"]))
        serialized_outs.append(num_to_var_int(len(out["script"])) + out["script"])
    outputs_hashed = dbl_sha256_list(serialized_outs)
    o.append(outputs_hashed)

    o.append(encode_4_bytes(txobj["locktime"]))

    return list_to_bytes(o)

def signature_form(tx, i, script, hashcode=SIGHASH_ALL):
    i, hashcode = int(i), int(hashcode)
    if isinstance(tx, string_or_bytes_types):
        tx = deserialize(tx)
    is_segwit = tx['ins'][i].get('segwit', False) or tx['ins'][i].get('new_segwit', False)
    newtx = copy.deepcopy(tx)
    for inp in newtx["ins"]:
        inp["script"] = ""
    newtx["ins"][i]["script"] = script
    if is_segwit or hashcode & 255 == SIGHASH_ALL + SIGHASH_FORKID:
        return uahf_digest(newtx, i)
    elif hashcode == SIGHASH_NONE:
        newtx["outs"] = []
    elif hashcode == SIGHASH_SINGLE:
        newtx["outs"] = newtx["outs"][:len(newtx["ins"])]
        for out in newtx["outs"][:len(newtx["ins"]) - 1]:
            out['value'] = 2**64 - 1
            out['script'] = ""
    elif hashcode == SIGHASH_ANYONECANPAY:
        newtx["ins"] = [newtx["ins"][i]]
    else:
        pass
    return serialize(newtx, include_witness=False)

# Making the actual signatures


def der_encode_sig(v, r, s):
    b1, b2 = safe_hexlify(encode(r, 256)), safe_hexlify(encode(s, 256))
    if len(b1) and b1[0] in '89abcdef':
        b1 = '00' + b1
    if len(b2) and b2[0] in '89abcdef':
        b2 = '00' + b2
    left = '02'+encode(len(b1)//2, 16, 2)+b1
    right = '02'+encode(len(b2)//2, 16, 2)+b2
    return '30'+encode(len(left+right)//2, 16, 2)+left+right

def der_decode_sig(sig):
    leftlen = decode(sig[6:8], 16)*2
    left = sig[8:8+leftlen]
    rightlen = decode(sig[10+leftlen:12+leftlen], 16)*2
    right = sig[12+leftlen:12+leftlen+rightlen]
    return (None, decode(left, 16), decode(right, 16))

def is_bip66(sig):
    """Checks hex DER sig for BIP66 consistency"""
    #https://raw.githubusercontent.com/bitcoin/bips/master/bip-0066.mediawiki
    #0x30  [total-len]  0x02  [R-len]  [R]  0x02  [S-len]  [S]  [sighash]
    sig = bytearray.fromhex(sig) if re.match('^[0-9a-fA-F]*$', sig) else bytearray(sig)
    if (sig[0] == 0x30) and (sig[1] == len(sig)-2):     # check if sighash is missing
            sig.extend(b"\1")		                   	# add SIGHASH_ALL for testing
    #assert (sig[-1] & 124 == 0) and (not not sig[-1]), "Bad SIGHASH value"
    
    if len(sig) < 9 or len(sig) > 73: return False
    if (sig[0] != 0x30): return False
    if (sig[1] != len(sig)-3): return False
    rlen = sig[3]
    if (5+rlen >= len(sig)): return False
    slen = sig[5+rlen]
    if (rlen + slen + 7 != len(sig)): return False
    if (sig[2] != 0x02): return False
    if (rlen == 0): return False
    if (sig[4] & 0x80): return False
    if (rlen > 1 and (sig[4] == 0x00) and not (sig[5] & 0x80)): return False
    if (sig[4+rlen] != 0x02): return False
    if (slen == 0): return False
    if (sig[rlen+6] & 0x80): return False
    if (slen > 1 and (sig[6+rlen] == 0x00) and not (sig[7+rlen] & 0x80)):
        return False
    return True

def txhash(tx, hashcode=None, wtxid=True):
    if isinstance(tx, str) and re.match('^[0-9a-fA-F]*$', tx):
        tx = changebase(tx, 16, 256)
    if not wtxid and is_segwit(tx):
        tx = serialize(deserialize(tx), include_witness=False)
    if hashcode:
        return dbl_sha256(from_string_to_bytes(tx) + encode(int(hashcode), 256, 4)[::-1])
    else:
        return safe_hexlify(bin_dbl_sha256(tx)[::-1])

def public_txhash(tx, hashcode=None):
    return txhash(tx, hashcode=hashcode, wtxid=False)

def bin_txhash(tx, hashcode=None):
    return binascii.unhexlify(txhash(tx, hashcode))


def ecdsa_tx_sign(tx, priv, hashcode=SIGHASH_ALL):
    bin_ = bin_txhash(tx, hashcode)
    rawsig = ecdsa_raw_sign(bin_, priv)
    return der_encode_sig(*rawsig)+encode(hashcode & 255, 16, 2)


def ecdsa_tx_verify(tx, sig, pub, hashcode=SIGHASH_ALL):
    return ecdsa_raw_verify(bin_txhash(tx, hashcode), der_decode_sig(sig), pub)


def ecdsa_tx_recover(tx, sig, hashcode=SIGHASH_ALL):
    z = bin_txhash(tx, hashcode)
    _, r, s = der_decode_sig(sig)
    left = ecdsa_raw_recover(z, (0, r, s))
    right = ecdsa_raw_recover(z, (1, r, s))
    return (encode_pubkey(left, 'hex'), encode_pubkey(right, 'hex'))

# Scripts

def mk_pubkey_script(addr):
    """
    Used in converting p2pkh address to input or output script
    """
    return '76a914' + b58check_to_hex(addr) + '88ac'

def mk_scripthash_script(addr):
    """
    Used in converting p2sh address to output script
    """
    return 'a914' + b58check_to_hex(addr) + '87'

def output_script_to_address(script, magicbyte=0):
    if script.startswith('76'):
        script = script[6:]
    else:
        script = script[4:]
    if script.endswith('88ac'):
        script = script[:-4]
    else:
        script = script[:-2]
    return bin_to_b58check(safe_from_hex(script), magicbyte=magicbyte)

def mk_p2w_scripthash_script(witver, witprog):
    """
    Used in converting a decoded pay to witness script hash address to output script
    """
    assert (0 <= witver <= 16)
    OP_n = witver + 0x50 if witver > 0 else 0
    return bytes_to_hex_string([OP_n]) + '14' + (bytes_to_hex_string(witprog))

def mk_p2wpkh_redeemscript(pubkey):
    """
    Used in converting public key to p2wpkh script
    """
    return '160014' + pubkey_to_hash_hex(pubkey)

def mk_p2wpkh_script(pubkey):
    """
    Used in converting public key to p2wpkh script
    """
    script = mk_p2wpkh_redeemscript(pubkey)[2:]
    return 'a914'+ hex_to_hash160(script) + '87'

def mk_p2wpkh_scriptcode(pubkey):
    """
    Used in signing for tx inputs
    """
    return '76a914' + pubkey_to_hash_hex(pubkey) + '88ac'

def p2wpkh_nested_script(pubkey):
    return '0014' + hash160(safe_from_hex(pubkey))

# Output script to address representation

def deserialize_script(script):
    if isinstance(script, str) and re.match('^[0-9a-fA-F]*$', script):
       return json_changebase(deserialize_script(binascii.unhexlify(script)),
                              lambda x: safe_hexlify(x))
    out, pos = [], 0
    while pos < len(script):
        code = from_byte_to_int(script[pos])
        if code == 0:
            out.append(None)
            pos += 1
        elif code <= 75:
            out.append(script[pos+1:pos+1+code])
            pos += 1 + code
        elif code <= 78:
            szsz = pow(2, code - 76)
            sz = decode(script[pos+szsz: pos:-1], 256)
            out.append(script[pos + 1 + szsz:pos + 1 + szsz + sz])
            pos += 1 + szsz + sz
        elif code <= 96:
            out.append(code - 80)
            pos += 1
        else:
            out.append(code)
            pos += 1
    return out


def serialize_script_unit(unit):
    if isinstance(unit, int):
        if unit < 16:
            return from_int_to_byte(unit + 80)
        else:
            return from_int_to_byte(unit)
    elif unit is None:
        return b'\x00'
    else:
        if len(unit) <= 75:
            return from_int_to_byte(len(unit))+unit
        elif len(unit) < 256:
            return from_int_to_byte(76)+from_int_to_byte(len(unit))+unit
        elif len(unit) < 65536:
            return from_int_to_byte(77)+encode(len(unit), 256, 2)[::-1]+unit
        else:
            return from_int_to_byte(78)+encode(len(unit), 256, 4)[::-1]+unit


if is_python2:
    def serialize_script(script):
        if json_is_base(script, 16):
            return binascii.hexlify(serialize_script(json_changebase(script,
                                    lambda x: binascii.unhexlify(x))))
        return ''.join(map(serialize_script_unit, script))
else:
    def serialize_script(script):
        if json_is_base(script, 16):
            return safe_hexlify(serialize_script(json_changebase(script,
                                    lambda x: binascii.unhexlify(x))))
        
        result = bytes()
        for b in map(serialize_script_unit, script):
            result += b if isinstance(b, bytes) else bytes(b, 'utf-8')
        return result

def mk_multisig_script(*args):  # [pubs],k or pub1,pub2...pub[n],M
    """
    :param args: List of public keys to used to create multisig and M, the number of signatures required to spend
    :return: multisig script
    """
    if isinstance(args[0], list):
        pubs, M = args[0], int(args[1])
    else:
        pubs = list(filter(lambda x: len(str(x)) >= 32, args))
        M = int(args[len(pubs)])
    N = len(pubs)
    return serialize_script([M]+pubs+[N]+[0xae])

# Signing and verifying


def verify_tx_input(tx, i, script, sig, pub):
    if re.match('^[0-9a-fA-F]*$', tx):
        tx = binascii.unhexlify(tx)
    if re.match('^[0-9a-fA-F]*$', script):
        script = binascii.unhexlify(script)
    if not re.match('^[0-9a-fA-F]*$', sig):
        sig = safe_hexlify(sig)
    hashcode = decode(sig[-2:], 16)
    modtx = signature_form(tx, int(i), script, hashcode)
    return ecdsa_tx_verify(modtx, sig, pub, hashcode)

def multisign(tx, i, script, pk, hashcode=SIGHASH_ALL):
    if isinstance(tx, dict):
        tx = serialize(tx)
    if re.match('^[0-9a-fA-F]*$', tx):
        tx = binascii.unhexlify(tx)
    if re.match('^[0-9a-fA-F]*$', script):
        script = binascii.unhexlify(script)
    modtx = signature_form(tx, i, script, hashcode)
    return ecdsa_tx_sign(modtx, pk, hashcode)


def apply_multisignatures(txobj, i, script, *args):
    # tx,i,script,sigs OR tx,i,script,sig1,sig2...,sig[n]
    sigs = args[0] if isinstance(args[0], list) else list(args)

    if isinstance(script, str) and re.match('^[0-9a-fA-F]*$', script):
        script = binascii.unhexlify(script)
    sigs = [binascii.unhexlify(x) if x[:2] == '30' else x for x in sigs]
    if not isinstance(txobj, dict):
        txobj = deserialize(txobj)
    if isinstance(txobj, str) and re.match('^[0-9a-fA-F]*$', txobj):
        return safe_hexlify(apply_multisignatures(binascii.unhexlify(txobj), i, script, sigs))

    # Not pushing empty elements on the top of the stack if passing no
    # script (in case of bare multisig inputs there is no script)
    script_blob = [] if script.__len__() == 0 else [script]

    txobj["ins"][i]["script"] = safe_hexlify(serialize_script([None]+sigs+script_blob))
    return serialize(txobj)


def is_inp(arg):
    return len(arg) > 64 or "output" in arg or "outpoint" in arg

def select(unspent, value):
    value = int(value)
    high = [u for u in unspent if u["value"] >= value]
    high.sort(key=lambda u: u["value"])
    low = [u for u in unspent if u["value"] < value]
    low.sort(key=lambda u: -u["value"])
    if len(high):
        return [high[0]]
    i, tv = 0, 0
    while tv < value and i < len(low):
        tv += low[i]["value"]
        i += 1
    if tv < value:
        raise Exception("Not enough funds")
    unspents = low[:i]
    actual_value = sum(unspent['value'] for unspent in unspents)
    return low[:i]



from wicc.wiccrpc.box.RpcManager import RpcManager


class Account(object):

    @classmethod
    def list(cls):
        """
        查询当前节点地址列表信息
        :return: 返回地址列表
        """
        data = {
            "method": "listaddr",
            "params": []
        }
        return RpcManager.request(data)["result"]

    @classmethod
    def generate_account(cls):
        """
        创建新地址
        :return: 新地址
        """
        data = {
            "method": "getnewaddress"
        }
        return RpcManager.request(data)["result"]["addr"]

    @classmethod
    def register_account(cls, address, fee=10001):
        """
        激活账户,新建的地址必须激活后才能作为交易发起方
        :param address:  需要激活的地址
        :param fee: 激活使用的小费 (单位sawi)
        :return: 激活的交易Hash
        """
        data = {
            "method": "registeraccounttx",
            "params": [address, fee]
        }
        return RpcManager.request(data)["result"]

    @classmethod
    def account_info(cls, address):
        """
        获取普通账户/合约账户地址详情
        :return: 账号信息
        """
        data = {
            "method": "getaccountinfo",
            "params": [address]
        }
        return RpcManager.request(data)["result"]

    @classmethod
    def get_private_key(cls, address):
        """
        获取地址对应的私钥
        :param address: 本钱包里普通账户地址
        :return:  私钥(WIF格式）
        """
        data = {
            "method": "dumpprivkey",
            "params": [address]
        }
        return RpcManager.request(data) #["privkey"]

    @classmethod
    def import_private_key(cls, private_key):
        """
        将私钥（由private key导出）导入钱包
        :param private_key: 普通账户地址私钥（WIF格式）
        :return: 普通账户地址
        """
        data = {
            "method": "importprivkey",
            "params": [private_key]
        }
        return RpcManager.request(data)["result"]

    @classmethod
    def validate_address(cls, address):
        """
        检查普通地址或者合约地址是否有效
        :param address:  普通账户/合约账户的地址/regid
        :return:
        """
        data = {
            "method": "validateaddr",
            "params": [address]
        }
        return RpcManager.request(data)["result"]["ret"]



import sys, re
import binascii
import os
import hashlib


if sys.version_info.major == 2:
    string_types = (str, unicode)
    string_or_bytes_types = string_types
    int_types = (int, float, long)

    # Base switching
    code_strings = {
        2: '01',
        10: '0123456789',
        16: '0123456789abcdef',
        32: 'abcdefghijklmnopqrstuvwxyz234567',
        58: '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz',
        256: ''.join([chr(x) for x in range(256)])
    }

    def bin_dbl_sha256(s):
        bytes_to_hash = from_string_to_bytes(s)
        return hashlib.sha256(hashlib.sha256(bytes_to_hash).digest()).digest()

    def lpad(msg, symbol, length):
        if len(msg) >= length:
            return msg
        return symbol * (length - len(msg)) + msg

    def get_code_string(base):
        if base in code_strings:
            return code_strings[base]
        else:
            raise ValueError("Invalid base!")

    def changebase(string, frm, to, minlen=0):
        if frm == to:
            return lpad(string, get_code_string(frm)[0], minlen)
        return encode(decode(string, frm), to, minlen)

    def bin_to_b58check(inp, magicbyte=0):
        if magicbyte == 0:
            inp = '\x00' + inp
        while magicbyte > 0:
            inp = chr(int(magicbyte % 256)) + inp
            magicbyte //= 256
        leadingzbytes = len(re.match('^\x00*', inp).group(0))
        checksum = bin_dbl_sha256(inp)[:4]
        return '1' * leadingzbytes + changebase(inp+checksum, 256, 58)

    def bytes_to_hex_string(b):
        return b.encode('hex')

    def safe_from_hex(s):
        return s.decode('hex')

    def from_int_representation_to_bytes(a):
        return str(a)

    def from_int_to_byte(a):
        return chr(a)

    def from_byte_to_int(a):
        return ord(a)

    def from_bytes_to_string(s):
        return s

    def from_string_to_bytes(a):
        return a

    def safe_hexlify(a):
        return binascii.hexlify(a)

    def encode(val, base, minlen=0):
        base, minlen = int(base), int(minlen)
        code_string = get_code_string(base)
        result = ""
        while val > 0:
            result = code_string[val % base] + result
            val //= base
        return code_string[0] * max(minlen - len(result), 0) + result

    def decode(string, base):
        base = int(base)
        code_string = get_code_string(base)
        result = 0
        if base == 16:
            string = string.lower()
        while len(string) > 0:
            result *= base
            result += code_string.find(string[0])
            string = string[1:]
        return result

    def random_string(x):
        return os.urandom(x)



#!/usr/bin/python
# -*- coding: utf-8 -*-
from configparser import ConfigParser
import base64
import os


def get_config(tag, key, config="config.ini"):
    cf = ConfigParser()
    config_path = os.path.join(os.getcwd(), "")
    cf_path = os.path.join(config_path, config)
    cf.read(cf_path)
    return cf.get(tag, key)


class Config(object):

    @classmethod
    def __content(cls):
        user = get_config("HOST", "username")
        password = get_config("HOST", "password")
        encode_str = "{}:{}".format(user, password).encode('utf-8')
        return "Basic " + base64.b64encode(encode_str).decode('utf-8')

    @classmethod
    def url(cls):
        return get_config("HOST", "host") + ":" + get_config("HOST", "port")

    @classmethod
    def header(cls):
        authorization = cls.__content()
        return {"Authorization": authorization, "content-Type": "application/json;"}




import logging

class LoggedClass(object):

    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.setLevel(logging.INFO)
        self.log_prefix = ''
        self.throttled = 0

    def log_info(self, msg, throttle=False):
        # Prevent annoying log messages by throttling them if there
        # are too many in a short period
        if throttle:
            self.throttled += 1
            if self.throttled > 3:
                return
            if self.throttled == 3:
                msg += ' (throttling later logs)'
        self.logger.info(self.log_prefix + msg)

    def log_warning(self, msg):
        self.logger.warning(self.log_prefix + msg)

    def log_error(self, msg):
        self.logger.error(self.log_prefix + msg)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from wicc.cryptos.coins.base import BaseCoin


class WaykiCoin(BaseCoin):
    coin_symbol = "WICC"
    display_name = "WaykiChain"
    enabled = True
    magicbyte = 73
    segwit_supported = True
    # script_magicbyte = 51
    script_magicbyte = 0x1a1d42ff
    hd_path = 99999
    wif_prefix = 153
    testnet_overrides = {
        'display_name': "WaykiChain Testnet",
        'coin_symbol': "wicc",
        'wif_prefix': 210,
        "hd_path": 99999, #99999, #999999,
        'magicbyte': 135,
        'script_magicbyte': 0xd75c7dfd
    }



# coding=utf-8
from wicc.wiccrpc.box.RpcManager import RpcManager


class Transaction(object):

    @classmethod
    def transfer(cls, sender, receiver, amount, fee=10000):
        """
        从源地址账户转账到目的地址账户，手续费默认为:10000sawi
        :param sender: 发送者地址
        :param receiver: 接受者地址
        :param amount: 发送金额(单位sawi)
        :param fee: 交易费用(单位sawi),最少10000sawi
        :return: 交易Hash
        """
        data = {
            "method": "sendtoaddresswithfee",
            "params": [sender, receiver, amount, fee]
        }
        return RpcManager.request(data)['result']['hash']

    @classmethod
    def generate_signature(cls, sender, receiver, amount, fee=10000):
        """
         生成交易签名
        :param sender: 发送者地址
        :param receiver: 接受者地址
        :param amount: 发送金额(单位sawi)
        :param fee: 交易费用(单位sawi),最少10000sawi
        :return: 交易签名
        """
        data = {
            "method": "gensendtoaddresstxraw",
            "params": [fee, amount, sender, receiver]
        }
        return RpcManager.request(data)['result']['rawtx']

    @classmethod
    def submit_signature(cls, signature):
        """
        提交交易至区块链
        :param signature: 已创建的交易签名字段，通过generate_signature生成
        :return: 交易哈希
        """
        data = {
            "method": "submittx",
            "params": [signature]
        }
        return RpcManager.request(data)['result']['hash']

    @classmethod
    def transaction_info(cls, txhash):
        """
        交易哈希查询交易详情
        :param txhash: 交易的Hash
        :return: 交易详情
        """
        data = {
            "method": "gettxdetail",
            "params": [txhash]
        }
        return RpcManager.request(data)['result']

    @classmethod
    def current_transaction_hash_list(cls):
        """
        获取当前节点交易Hash列表
        :return: ConfirmTx: 已确认的交易哈希列表, UnConfirmTx 未确认的交易哈希列表
        """
        data = {
            "method": "listtx"
        }
        return RpcManager.request(data)['result']

    @classmethod
    def current_unconfirmed_transaction_list(cls):
        """
        获取当前节点未被确认的交易列表
        :return: 当前节点未被确认的交易列表
        """
        data = {
            "method": "listunconfirmedtx"
        }
        return RpcManager.request(data)['result']["UnConfirmTx"]

    @classmethod
    def current_all_transaction_list(cls, count):
        """
        获取当前节点交易详情
        :param count: 条数
        :return: UnConfirmed & Confirmed
        """
        data = {
            "method": "getalltxinfo",
            "params": count
        }
        return RpcManager.request(data)['result']

    @classmethod
    def decode_transaction_info(cls, hexstring):
        """
        获取交易详情
        :param hexstring: 条数
        :return: UnConfirmed & Confirmed
        """
        data = {
            "method": "getalltxinfo"
        }
        return RpcManager.request(data)['result']


if __name__ == '__main__':
    Transaction.all_transfer_list(10)



from . import base_insight as insight

#Docs: https://github.com/bitpay/insight-api

base_url = "https://bch-insight.bitpay.com/api"

def unspent(*args):
    return insight.unspent(base_url, *args)

def pushtx(*args):
    return insight.pushtx(base_url, *args)

def history(*args):
    return insight.history(base_url, *args)


#!/usr/bin/python
# -*- coding: utf-8 -*-
s = """
가격
가끔
가난
가능
가득
가르침
가뭄
가방
가상
가슴
가운데
가을
가이드
가입
가장
가정
가족
가죽
각오
각자
간격
간부
간섭
간장
간접
간판
갈등
갈비
갈색
갈증
감각
감기
감소
감수성
감자
감정
갑자기
강남
강당
강도
강력히
강변
강북
강사
강수량
강아지
강원도
강의
강제
강조
같이
개구리
개나리
개방
개별
개선
개성
개인
객관적
거실
거액
거울
거짓
거품
걱정
건강
건물
건설
건조
건축
걸음
검사
검토
게시판
게임
겨울
견해
결과
결국
결론
결석
결승
결심
결정
결혼
경계
경고
경기
경력
경복궁
경비
경상도
경영
경우
경쟁
경제
경주
경찰
경치
경향
경험
계곡
계단
계란
계산
계속
계약
계절
계층
계획
고객
고구려
고궁
고급
고등학생
고무신
고민
고양이
고장
고전
고집
고춧가루
고통
고향
곡식
골목
골짜기
골프
공간
공개
공격
공군
공급
공기
공동
공무원
공부
공사
공식
공업
공연
공원
공장
공짜
공책
공통
공포
공항
공휴일
과목
과일
과장
과정
과학
관객
관계
관광
관념
관람
관련
관리
관습
관심
관점
관찰
광경
광고
광장
광주
괴로움
굉장히
교과서
교문
교복
교실
교양
교육
교장
교직
교통
교환
교훈
구경
구름
구멍
구별
구분
구석
구성
구속
구역
구입
구청
구체적
국가
국기
국내
국립
국물
국민
국수
국어
국왕
국적
국제
국회
군대
군사
군인
궁극적
권리
권위
권투
귀국
귀신
규정
규칙
균형
그날
그냥
그늘
그러나
그룹
그릇
그림
그제서야
그토록
극복
극히
근거
근교
근래
근로
근무
근본
근원
근육
근처
글씨
글자
금강산
금고
금년
금메달
금액
금연
금요일
금지
긍정적
기간
기관
기념
기능
기독교
기둥
기록
기름
기법
기본
기분
기쁨
기숙사
기술
기억
기업
기온
기운
기원
기적
기준
기침
기혼
기획
긴급
긴장
길이
김밥
김치
김포공항
깍두기
깜빡
깨달음
깨소금
껍질
꼭대기
꽃잎
나들이
나란히
나머지
나물
나침반
나흘
낙엽
난방
날개
날씨
날짜
남녀
남대문
남매
남산
남자
남편
남학생
낭비
낱말
내년
내용
내일
냄비
냄새
냇물
냉동
냉면
냉방
냉장고
넥타이
넷째
노동
노란색
노력
노인
녹음
녹차
녹화
논리
논문
논쟁
놀이
농구
농담
농민
농부
농업
농장
농촌
높이
눈동자
눈물
눈썹
뉴욕
느낌
늑대
능동적
능력
다방
다양성
다음
다이어트
다행
단계
단골
단독
단맛
단순
단어
단위
단점
단체
단추
단편
단풍
달걀
달러
달력
달리
닭고기
담당
담배
담요
담임
답변
답장
당근
당분간
당연히
당장
대규모
대낮
대단히
대답
대도시
대략
대량
대륙
대문
대부분
대신
대응
대장
대전
대접
대중
대책
대출
대충
대통령
대학
대한민국
대합실
대형
덩어리
데이트
도대체
도덕
도둑
도망
도서관
도심
도움
도입
도자기
도저히
도전
도중
도착
독감
독립
독서
독일
독창적
동화책
뒷모습
뒷산
딸아이
마누라
마늘
마당
마라톤
마련
마무리
마사지
마약
마요네즈
마을
마음
마이크
마중
마지막
마찬가지
마찰
마흔
막걸리
막내
막상
만남
만두
만세
만약
만일
만점
만족
만화
많이
말기
말씀
말투
맘대로
망원경
매년
매달
매력
매번
매스컴
매일
매장
맥주
먹이
먼저
먼지
멀리
메일
며느리
며칠
면담
멸치
명단
명령
명예
명의
명절
명칭
명함
모금
모니터
모델
모든
모범
모습
모양
모임
모조리
모집
모퉁이
목걸이
목록
목사
목소리
목숨
목적
목표
몰래
몸매
몸무게
몸살
몸속
몸짓
몸통
몹시
무관심
무궁화
무더위
무덤
무릎
무슨
무엇
무역
무용
무조건
무지개
무척
문구
문득
문법
문서
문제
문학
문화
물가
물건
물결
물고기
물론
물리학
물음
물질
물체
미국
미디어
미사일
미술
미역
미용실
미움
미인
미팅
미혼
민간
민족
민주
믿음
밀가루
밀리미터
밑바닥
바가지
바구니
바나나
바늘
바닥
바닷가
바람
바이러스
바탕
박물관
박사
박수
반대
반드시
반말
반발
반성
반응
반장
반죽
반지
반찬
받침
발가락
발걸음
발견
발달
발레
발목
발바닥
발생
발음
발자국
발전
발톱
발표
밤하늘
밥그릇
밥맛
밥상
밥솥
방금
방면
방문
방바닥
방법
방송
방식
방안
방울
방지
방학
방해
방향
배경
배꼽
배달
배드민턴
백두산
백색
백성
백인
백제
백화점
버릇
버섯
버튼
번개
번역
번지
번호
벌금
벌레
벌써
범위
범인
범죄
법률
법원
법적
법칙
베이징
벨트
변경
변동
변명
변신
변호사
변화
별도
별명
별일
병실
병아리
병원
보관
보너스
보라색
보람
보름
보상
보안
보자기
보장
보전
보존
보통
보편적
보험
복도
복사
복숭아
복습
볶음
본격적
본래
본부
본사
본성
본인
본질
볼펜
봉사
봉지
봉투
부근
부끄러움
부담
부동산
부문
부분
부산
부상
부엌
부인
부작용
부장
부정
부족
부지런히
부친
부탁
부품
부회장
북부
북한
분노
분량
분리
분명
분석
분야
분위기
분필
분홍색
불고기
불과
불교
불꽃
불만
불법
불빛
불안
불이익
불행
브랜드
비극
비난
비닐
비둘기
비디오
비로소
비만
비명
비밀
비바람
비빔밥
비상
비용
비율
비중
비타민
비판
빌딩
빗물
빗방울
빗줄기
빛깔
빨간색
빨래
빨리
사건
사계절
사나이
사냥
사람
사랑
사립
사모님
사물
사방
사상
사생활
사설
사슴
사실
사업
사용
사월
사장
사전
사진
사촌
사춘기
사탕
사투리
사흘
산길
산부인과
산업
산책
살림
살인
살짝
삼계탕
삼국
삼십
삼월
삼촌
상관
상금
상대
상류
상반기
상상
상식
상업
상인
상자
상점
상처
상추
상태
상표
상품
상황
새벽
색깔
색연필
생각
생명
생물
생방송
생산
생선
생신
생일
생활
서랍
서른
서명
서민
서비스
서양
서울
서적
서점
서쪽
서클
석사
석유
선거
선물
선배
선생
선수
선원
선장
선전
선택
선풍기
설거지
설날
설렁탕
설명
설문
설사
설악산
설치
설탕
섭씨
성공
성당
성명
성별
성인
성장
성적
성질
성함
세금
세미나
세상
세월
세종대왕
세탁
센터
센티미터
셋째
소규모
소극적
소금
소나기
소년
소득
소망
소문
소설
소속
소아과
소용
소원
소음
소중히
소지품
소질
소풍
소형
속담
속도
속옷
손가락
손길
손녀
손님
손등
손목
손뼉
손실
손질
손톱
손해
솔직히
솜씨
송아지
송이
송편
쇠고기
쇼핑
수건
수년
수단
수돗물
수동적
수면
수명
수박
수상
수석
수술
수시로
수업
수염
수영
수입
수준
수집
수출
수컷
수필
수학
수험생
수화기
숙녀
숙소
숙제
순간
순서
순수
순식간
순위
숟가락
술병
술집
숫자
스님
스물
스스로
스승
스웨터
스위치
스케이트
스튜디오
스트레스
스포츠
슬쩍
슬픔
습관
습기
승객
승리
승부
승용차
승진
시각
시간
시골
시금치
시나리오
시댁
시리즈
시멘트
시민
시부모
시선
시설
시스템
시아버지
시어머니
시월
시인
시일
시작
시장
시절
시점
시중
시즌
시집
시청
시합
시험
식구
식기
식당
식량
식료품
식물
식빵
식사
식생활
식초
식탁
식품
신고
신규
신념
신문
신발
신비
신사
신세
신용
신제품
신청
신체
신화
실감
실내
실력
실례
실망
실수
실습
실시
실장
실정
실질적
실천
실체
실컷
실태
실패
실험
실현
심리
심부름
심사
심장
심정
심판
쌍둥이
씨름
씨앗
아가씨
아나운서
아드님
아들
아쉬움
아스팔트
아시아
아울러
아저씨
아줌마
아직
아침
아파트
아프리카
아픔
아홉
아흔
악기
악몽
악수
안개
안경
안과
안내
안녕
안동
안방
안부
안주
알루미늄
알코올
암시
암컷
압력
앞날
앞문
애인
애정
액수
앨범
야간
야단
야옹
약간
약국
약속
약수
약점
약품
약혼녀
양념
양력
양말
양배추
양주
양파
어둠
어려움
어른
어젯밤
어쨌든
어쩌다가
어쩐지
언니
언덕
언론
언어
얼굴
얼른
얼음
얼핏
엄마
업무
업종
업체
엉덩이
엉망
엉터리
엊그제
에너지
에어컨
엔진
여건
여고생
여관
여군
여권
여대생
여덟
여동생
여든
여론
여름
여섯
여성
여왕
여인
여전히
여직원
여학생
여행
역사
역시
역할
연결
연구
연극
연기
연락
연설
연세
연속
연습
연애
연예인
연인
연장
연주
연출
연필
연합
연휴
열기
열매
열쇠
열심히
열정
열차
열흘
염려
엽서
영국
영남
영상
영양
영역
영웅
영원히
영하
영향
영혼
영화
옆구리
옆방
옆집
예감
예금
예방
예산
예상
예선
예술
예습
예식장
예약
예전
예절
예정
예컨대
옛날
오늘
오락
오랫동안
오렌지
오로지
오른발
오븐
오십
오염
오월
오전
오직
오징어
오페라
오피스텔
오히려
옥상
옥수수
온갖
온라인
온몸
온종일
온통
올가을
올림픽
올해
옷차림
와이셔츠
와인
완성
완전
왕비
왕자
왜냐하면
왠지
외갓집
외국
외로움
외삼촌
외출
외침
외할머니
왼발
왼손
왼쪽
요금
요일
요즘
요청
용기
용서
용어
우산
우선
우승
우연히
우정
우체국
우편
운동
운명
운반
운전
운행
울산
울음
움직임
웃어른
웃음
워낙
원고
원래
원서
원숭이
원인
원장
원피스
월급
월드컵
월세
월요일
웨이터
위반
위법
위성
위원
위험
위협
윗사람
유난히
유럽
유명
유물
유산
유적
유치원
유학
유행
유형
육군
육상
육십
육체
은행
음력
음료
음반
음성
음식
음악
음주
의견
의논
의문
의복
의식
의심
의외로
의욕
의원
의학
이것
이곳
이념
이놈
이달
이대로
이동
이렇게
이력서
이론적
이름
이민
이발소
이별
이불
이빨
이상
이성
이슬
이야기
이용
이웃
이월
이윽고
이익
이전
이중
이튿날
이틀
이혼
인간
인격
인공
인구
인근
인기
인도
인류
인물
인생
인쇄
인연
인원
인재
인종
인천
인체
인터넷
인하
인형
일곱
일기
일단
일대
일등
일반
일본
일부
일상
일생
일손
일요일
일월
일정
일종
일주일
일찍
일체
일치
일행
일회용
임금
임무
입대
입력
입맛
입사
입술
입시
입원
입장
입학
자가용
자격
자극
자동
자랑
자부심
자식
자신
자연
자원
자율
자전거
자정
자존심
자판
작가
작년
작성
작업
작용
작은딸
작품
잔디
잔뜩
잔치
잘못
잠깐
잠수함
잠시
잠옷
잠자리
잡지
장관
장군
장기간
장래
장례
장르
장마
장면
장모
장미
장비
장사
장소
장식
장애인
장인
장점
장차
장학금
재능
재빨리
재산
재생
재작년
재정
재채기
재판
재학
재활용
저것
저고리
저곳
저녁
저런
저렇게
저번
저울
저절로
저축
적극
적당히
적성
적용
적응
전개
전공
전기
전달
전라도
전망
전문
전반
전부
전세
전시
전용
전자
전쟁
전주
전철
전체
전통
전혀
전후
절대
절망
절반
절약
절차
점검
점수
점심
점원
점점
점차
접근
접시
접촉
젓가락
정거장
정도
정류장
정리
정말
정면
정문
정반대
정보
정부
정비
정상
정성
정오
정원
정장
정지
정치
정확히
제공
제과점
제대로
제목
제발
제법
제삿날
제안
제일
제작
제주도
제출
제품
제한
조각
조건
조금
조깅
조명
조미료
조상
조선
조용히
조절
조정
조직
존댓말
존재
졸업
졸음
종교
종로
종류
종소리
종업원
종종
종합
좌석
죄인
주관적
주름
주말
주머니
주먹
주문
주민
주방
주변
주식
주인
주일
주장
주전자
주택
준비
줄거리
줄기
줄무늬
중간
중계방송
중국
중년
중단
중독
중반
중부
중세
중소기업
중순
중앙
중요
중학교
즉석
즉시
즐거움
증가
증거
증권
증상
증세
지각
지갑
지경
지극히
지금
지급
지능
지름길
지리산
지방
지붕
지식
지역
지우개
지원
지적
지점
지진
지출
직선
직업
직원
직장
진급
진동
진로
진료
진리
진짜
진찰
진출
진통
진행
질문
질병
질서
짐작
집단
집안
집중
짜증
찌꺼기
차남
차라리
차량
차림
차별
차선
차츰
착각
찬물
찬성
참가
참기름
참새
참석
참여
참외
참조
찻잔
창가
창고
창구
창문
창밖
창작
창조
채널
채점
책가방
책방
책상
책임
챔피언
처벌
처음
천국
천둥
천장
천재
천천히
철도
철저히
철학
첫날
첫째
청년
청바지
청소
청춘
체계
체력
체온
체육
체중
체험
초등학생
초반
초밥
초상화
초순
초여름
초원
초저녁
초점
초청
초콜릿
촛불
총각
총리
총장
촬영
최근
최상
최선
최신
최악
최종
추석
추억
추진
추천
추측
축구
축소
축제
축하
출근
출발
출산
출신
출연
출입
출장
출판
충격
충고
충돌
충분히
충청도
취업
취직
취향
치약
친구
친척
칠십
칠월
칠판
침대
침묵
침실
칫솔
칭찬
카메라
카운터
칼국수
캐릭터
캠퍼스
캠페인
커튼
컨디션
컬러
컴퓨터
코끼리
코미디
콘서트
콜라
콤플렉스
콩나물
쾌감
쿠데타
크림
큰길
큰딸
큰소리
큰아들
큰어머니
큰일
큰절
클래식
클럽
킬로
타입
타자기
탁구
탁자
탄생
태권도
태양
태풍
택시
탤런트
터널
터미널
테니스
테스트
테이블
텔레비전
토론
토마토
토요일
통계
통과
통로
통신
통역
통일
통장
통제
통증
통합
통화
퇴근
퇴원
퇴직금
튀김
트럭
특급
특별
특성
특수
특징
특히
튼튼히
티셔츠
파란색
파일
파출소
판결
판단
판매
판사
팔십
팔월
팝송
패션
팩스
팩시밀리
팬티
퍼센트
페인트
편견
편의
편지
편히
평가
평균
평생
평소
평양
평일
평화
포스터
포인트
포장
포함
표면
표정
표준
표현
품목
품질
풍경
풍속
풍습
프랑스
프린터
플라스틱
피곤
피망
피아노
필름
필수
필요
필자
필통
핑계
하느님
하늘
하드웨어
하룻밤
하반기
하숙집
하순
하여튼
하지만
하천
하품
하필
학과
학교
학급
학기
학년
학력
학번
학부모
학비
학생
학술
학습
학용품
학원
학위
학자
학점
한계
한글
한꺼번에
한낮
한눈
한동안
한때
한라산
한마디
한문
한번
한복
한식
한여름
한쪽
할머니
할아버지
할인
함께
함부로
합격
합리적
항공
항구
항상
항의
해결
해군
해답
해당
해물
해석
해설
해수욕장
해안
핵심
핸드백
햄버거
햇볕
햇살
행동
행복
행사
행운
행위
향기
향상
향수
허락
허용
헬기
현관
현금
현대
현상
현실
현장
현재
현지
혈액
협력
형부
형사
형수
형식
형제
형태
형편
혜택
호기심
호남
호랑이
호박
호텔
호흡
혹시
홀로
홈페이지
홍보
홍수
홍차
화면
화분
화살
화요일
화장
화학
확보
확인
확장
확정
환갑
환경
환영
환율
환자
활기
활동
활발히
활용
활짝
회견
회관
회복
회색
회원
회장
회전
횟수
횡단보도
효율적
후반
후춧가루
훈련
훨씬
휴식
휴일
흉내
흐름
흑백
흑인
흔적
흔히
흥미
흥분
희곡
희망
희생
흰색
힘껏
"""
words_list = [x for x in s.split("\n") if x is not ""]


#!/usr/bin/env python3
#
# Copyright (c) 2016, Neil Booth
#
# All rights reserved.
#
# See the file "LICENCE" for information about the copyright
# and warranty status of this software.

'''Script to send RPC commands to a running ElectrumX server.'''


import asyncio
import json
import random
import os
from functools import partial

from .jsonrpc import JSONSession, JSONRPCv2


class RPCClient(JSONSession):

    def __init__(self):
        super().__init__(version=JSONRPCv2)
        self.max_send = 0
        self.max_buffer_size = 5*10**6
        self.result = {}

    async def wait_for_response(self, id_):
        from datetime import datetime
        now = datetime.now()
        await self.items_events[id_].wait()
        print(id_, "event raised:", datetime.now() - now)
        await self.process_pending_items()
        self.items_events[id_].clear()
        del self.items_events[id_]
        return self.result.pop(id_)

    def send_rpc_request(self, method, params):
        handler = partial(self.handle_response, method, params)
        return self.send_request(handler, method, params)

    def handle_response(self, method, params, id_, data, error):
        self.result[id_] = {'data': data, 'error': error, 'method': method, 'params': params}


def read_json(filename, default):
    path = os.path.join(os.path.dirname(__file__), 'servers', filename)
    try:
        with open(path, 'r') as f:
            r = json.loads(f.read())
    except:
        r = default
    return r

class ElectrumXClient(RPCClient):

    def __init__(self, server_file="bitcoin.json", servers=(), host=None, port=50001, timeout=15, max_servers=5, loop=None):
        super().__init__()
        if loop:
            self.loop = loop
        else:
            self.loop = asyncio.get_event_loop()
        self.timeout = timeout
        self.failed_hosts = []
        self.max_servers = max_servers
        if servers:
            self.servers = servers
        else:
            self.servers = read_json(server_file, {})
        self.host = host
        self.port = port
        self.rpc_client = None
        if not self.host:
            self.host, self.port = self.choose_random_server()
        self.connect_to_server()

    def choose_random_server(self):
        host = random.choice(list(self.servers.keys()))
        try:
            return host, self.servers[host]['t']
        except KeyError:
            del self.servers[host]
            return self.choose_random_server()

    def connect_to_server(self):
        print(self.host, self.port)
        try:
            coro = self.loop.create_connection(RPCClient, self.host, self.port)
            transport, self.rpc_client = self.loop.run_until_complete(coro)
        except OSError:
            self.change_server()

    def change_server(self):
        if self.rpc_client:
            self.rpc_client.close()
        if self.host not in self.failed_hosts:
            self.failed_hosts.append(self.host)
        if len(self.failed_hosts) >= self.max_servers:
            raise Exception("Attempted to connect to %s servers but failed" % len(self.failed_hosts))
        while self.host in self.failed_hosts:
            self.host, self.port = self.choose_random_server()
        self.connect_to_server()

    def rpc_multiple_send_and_wait(self, requests):
        from datetime import datetime
        coroutines = []
        for request in requests:
            method, params = request
            try:
                now = datetime.now()
                id_ = self.rpc_client.send_rpc_request(method, params)
                print(id_, "Request sent:", datetime.now() - now)
                try:
                    coro = self.rpc_client.wait_for_response(id_)
                    coroutines.append(asyncio.wait_for(coro, self.timeout))
                except asyncio.TimeoutError:
                    self.change_server()
                    return self.rpc_multiple_send_and_wait(requests)
            except OSError:
                self.change_server()
                return self.rpc_multiple_send_and_wait(requests)
        now = datetime.now()
        values = self.loop.run_until_complete(asyncio.gather(*coroutines))
        print("Values gathered:", datetime.now() - now)
        self.failed_hosts = []
        return values

    def unspent(self, *addrs):
        requests = [("blockchain.address.listunspent", [addr]) for addr in addrs]
        results = self.rpc_multiple_send_and_wait(requests)
        unspents = []
        for i, result in enumerate(results):
            if result['error']:
                raise Exception(result['error'])
            unspent_for_addr = result['data']
            addr = result['params'][0]
            for u in unspent_for_addr:
                u['address'] = addr
                unspents.append(u)
        return unspents

    


from .base import BaseCoin
from ..explorers import sochain


class Litecoin(BaseCoin):
    coin_symbol = "LTC"
    display_name = "Litecoin"
    segwit_supported = True
    magicbyte = 48
    #script_magicbyte = 50 #Supposed to be new magicbyte
    script_magicbyte = 5 #Old magicbyte still recognised by explorers
    wif_prefix = 0xb0
    segwit_hrp = "ltc1"
    hd_path = 2
    explorer = sochain
    testnet_overrides = {
        'display_name': "Litecoin Testnet",
        'coin_symbol': "LTCTEST",
        'magicbyte': 111,
        #script_magicbyte: 58   #Supposed to be new magicbyte
        'script_magicbyte': 196, #Old magicbyte still recognised by explorers,
        'segwit_hrp': "tltc1",
        'hd_path': 1,
        'xpriv_prefix': 0x04358394,
        'xpub_prefix': 0x043587cf
    }



#!/usr/bin/python
# -*- coding: utf-8 -*-

from wicc.transactions import *
from cryptos import *


def number_to_var_int(number):
    batch = 0
    num = number
    while True:
        batch += 1
        if num <= 0x7F:
            break
        num = (num >> 7) - 1

    tmp_list = [0 for _ in range(0, int((batch * 8 + 6) / 7))]
    index = 0
    while True:
        cache = 0
        if index == 0:
            cache = 0x00
        else:
            cache = 0x80
        tmp_list[index] = number & 0x7F | cache
        if number <= 0x7F:
            break
        number = (number >> 7) - 1
        index += 1
    return bytes(tmp_list[batch - 1::-1])


def number_str_to_bytes(str_):
    t = str_.upper()
    p = re.compile('.{1,2}')
    return bytes([int(x, 16) for x in p.findall(t)])


class Serializer(object):
    """
    序列化参数
    """
    def __init__(self):
        self.out: [bytes] = []

    def ser_version(self, version):
        """
        :param version: 版本号
        """
        self.out.append(number_to_var_int(version))
        return self

    def ser_tx_type(self, type):
        """
        :param type: 交易类型
        """
        self.out.append(number_to_var_int(type))
        return self

    def ser_valid_height(self, height):
        """
        :param height: 区块高度
        """
        self.out.append(number_to_var_int(height))
        return self

    def ser_regid_or_public_key(self, regid, public_key):
        """
        :param regid: 用户的Id
        :param public_key: 账户的公钥
        """
        result = regid.split("-")
        if len(result) == 2:
            height = number_to_var_int(int(result[0]))
            index = number_to_var_int(int(result[1]))
            self.out.append(num_to_var_int(len(height) + len(index)))
            self.out.append(height)
            self.out.append(index)
        else:
            public_key_bytes = from_string_to_bytes(public_key)
            self.out.append(num_to_var_int(len(public_key_bytes)))
            self.out.append(public_key_bytes)
        return self

    def ser_regid(self, regid):
        """
        :param regid: register id
        """
        result = regid.split("-")
        height = number_to_var_int(int(result[0]))
        index = number_to_var_int(int(result[1]))
        self.out.append(num_to_var_int(len(height) + len(index)))
        self.out.append(height)
        self.out.append(index)
        return self

    def ser_desert_address(self, desert_address):
        """
            :param desert_address: 接受地址
            """
        address_bin = b58check_to_bin(desert_address)
        self.out.append(num_to_var_int(len(address_bin)))
        self.out.append(address_bin)
        return self

    def ser_fee(self, fee_amount, fee_coin_symbol):
        """
        :param fee_amount: 矿工费
        :param fee_coin_symbol: 矿工费币种
        """
        coin_symbol_byte = from_string_to_bytes(fee_coin_symbol)
        self.out.append(num_to_var_int(len(coin_symbol_byte)))
        self.out.append(coin_symbol_byte)
        self.out.append(number_to_var_int(fee_amount))
        return self

    def ser_fee_amount(self, amount):
        self.out.append(number_to_var_int(amount))
        return self

    def ser_coin_symbol(self, coin_symbol):
        coin_symbol_byte = from_string_to_bytes(coin_symbol)
        self.out.append(num_to_var_int(len(coin_symbol_byte)))
        self.out.append(coin_symbol_byte)
        return self

    def ser_coin_amount(self, amount):
        self.out.append(number_to_var_int(amount))
        return self

    def ser_order_id(self, order_id):
        self.out.append(number_str_to_bytes(order_id)[::-1])
        return self

    def ser_cdp_id(self, order_id):
        self.out.append(number_str_to_bytes(order_id)[::-1])
        return self

    def ser_cdp_stake_list(self, asset_list: [CdpStakeAsset]):
        self.out.append(num_to_var_int(len(asset_list)))
        for asset in asset_list:
            self.ser_coin_symbol(asset.symbol)
            self.ser_coin_amount(asset.amount)
        return self

    def ser_cdp_redeem_list(self, redeem_list: [CdpRedeemAsset]):
        self.out.append(num_to_var_int(len(redeem_list)))
        for asset in redeem_list:
            self.ser_coin_symbol(asset.symbol)
            self.ser_coin_amount(asset.amount)
        return self

    def ser_asset_type_and_value(self, type_, value):
        if type_ == 1:
            self.out.append(number_to_var_int(1))
            self.ser_regid(value)
        elif type_ == 2:
            self.out.append(number_to_var_int(2))
            self.ser_memo(value)
        elif type_ == 3:
            self.out.append(number_to_var_int(3))
            self.out.append(number_to_var_int(value))
        return self

    def ser_amount(self, amount, coin_symbol):
        """
        :param amount: 数量
        :param coin_symbol: 币种
        """
        coin_symbol_byte = from_string_to_bytes(coin_symbol)
        self.out.append(num_to_var_int(len(coin_symbol_byte)))
        self.out.append(coin_symbol_byte)
        self.out.append(number_to_var_int(amount))
        return self

    def ser_transfer(self, transfer_list: [Transfer]):
        """
        :param transfer_list: 转账列表
        """
        self.out.append(num_to_var_int(len(transfer_list)))
        for tf in transfer_list:
            address_bin = b58check_to_bin(tf.desert_address)
            self.out.append(num_to_var_int(len(address_bin)))
            self.out.append(address_bin)

            coin_symbol_byte = from_string_to_bytes(tf.pay_coin_symbol)
            self.out.append(num_to_var_int(len(coin_symbol_byte)))
            self.out.append(coin_symbol_byte)
            self.out.append(number_to_var_int(tf.pay_amount))
        return self

    def ser_memo(self, memo):
        """
        :param memo: 转账备注
        """
        memo_bytes = from_string_to_bytes(memo)
        self.out.append(num_to_var_int(len(memo_bytes)))
        self.out.append(memo_bytes)
        return self

    def ser_signature(self, signature):
        """
        :param signature: 签名
        """
        signature_bytes = number_str_to_bytes(signature)
        self.out.append(num_to_var_int(len(signature_bytes)))
        self.out.append(signature_bytes)
        return self

    def ser_contract_message(self, message):
        msg_bytes = number_str_to_bytes(message)
        self.out.append(num_to_var_int(len(msg_bytes)))
        self.out.append(msg_bytes)
        return self

    def to_bytes(self):
        return list_to_bytes(self.out)

    def to_hex_string(self):
        return bytes_to_hex_string(list_to_bytes(self.out))

    def to_message(self):
        return dbl_sha256_list(self.out)

    @classmethod
    def string_to_bytes(cls, string):
        return from_string_to_bytes(string)


class Stamper(object):

    @classmethod
    def stamp(cls, message, private_key):
        """
        :param message: 需要签名的消息
        :param private_key: 私钥
        :return: 签名
        """
        return der_encode_sig(*ecdsa_raw_sign(message, private_key))




from .base import BaseCoin
from ..explorers import dash_siampm

class Dash(BaseCoin):
    coin_symbol = "DASH"
    display_name = "Dash"
    segwit_supported = False
    magicbyte = 76
    script_magicbyte = 16
    wif_prefix = 0xcc
    hd_path = 5
    explorer = dash_siampm
    testnet_overrides = {
        'display_name': "Dash Testnet",
        'coin_symbol': "DASHTEST",
        'magicbyte': 140,
        'script_magicbyte': 19,
        'hd_path': 1,
        'xpriv_prefix': 0x04358394,
        'xpub_prefix': 0x043587cf
    }
